#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import logging
import logging.config
import commands
import os
 
from PyQt4 import QtGui, QtCore

from hga.controlgrupos.servidor.appcoreprofe import AppCoreProfe
from hga.controlgrupos.servidor.settingsservidor import GUI_SERVIDOR

from hga.controlstreaming.servidor.stream_server import stream_Servidor

from hga.controlgrupos.cliente.appcorealumno import AppCoreAlumno
from hga.controlgrupos.cliente.settingscliente import GUI_CLIENTE

from hga.controlstreaming.cliente.stream_cliente import stream_Cliente

from hga.controlgrupos.common.settings import LOGGING_CONFIG_FILE
from hga.controlgrupos.common.user_info import get_user_name

from hga.gestioncontrolgrupos import GestionControlGrupos
from hga.controldesktop.controldesktop import ControlDesktop
from hga.controldesktop.gestioncontroldesktop import GestionControlDesktop
from hga.controleditor.gestioncontroleditor import GestionControlEditor
from hga.controleditor.controleditor_server import ControlEditor_Server


from hga.gui.servidor.sgestion import SGestion
from hga.gui.alumno.sguialumno import SGuiAlumno


class AppCore:
	def __init__(self,interfaz):
		self.appgui = None
		self.qtapp = None
		self.gui_handlers = None
		self.pidCliente = None

		#manejadores del stream
		self.stream_handlers = None

		self.appcore_handlers = {
            "EXIT": self.cerrar_aplicacion,
        }

		self.gui = None
		self.core = None
		self.stream = None
		self.setup_interfaz()

		if interfaz == GUI_SERVIDOR: #Según la app que se quiera lanzar
			self.setup_servidor()
		elif interfaz == GUI_CLIENTE:
			self.setup_cliente()

	def iniciar(self):
		self.gui.show()
		self.qtapp.exec_()

	def iniciar_cliente(self):
		sys.exit(self.qtapp.exec_())

	def setup_servidor(self):
		self.acceso_desktop = ControlDesktop()
		self.CE_Server = ControlEditor_Server()

		vnc_handler = self.acceso_desktop.controldesktop_handler()
		ce_handlers = self.CE_Server.controleditor_handlers()

		self.gui = SGestion()
		self.core = AppCoreProfe(self.gui, vnc_handler, ce_handlers)
		self.stream = stream_Servidor(self.core)
		self.gcGrupos = GestionControlGrupos(self.gui, self.core, self.stream)
		
		self.gcDesktop = GestionControlDesktop(self.core, self.acceso_desktop)
		self.gcEditor = GestionControlEditor(self.core, self.CE_Server)
		
		# Obtenemos los handlers para la GUI
		self.gui_handlers = self.gcGrupos.gui_handlers_servidor()
		self.controldesktop_gui_handlers = self.gcDesktop.gui_handlers_controldesktop()
		self.controleditor_gui_handlers = self.gcEditor.gui_handlers_CEServer()
		self.gui.set_handlers(self.gui_handlers, self.controldesktop_gui_handlers, self.controleditor_gui_handlers)
		#Internacionalización
		self.appgui = "sgui_profesor_"
		self.i10n()

	def setup_cliente(self):
		self.appgui = "sgui_alumno_"
		self.i10n() # Internacionalización
		self.qtapp.setQuitOnLastWindowClosed(False)
		self.gui = SGuiAlumno()
		self.stream = stream_Cliente()
		self.stream_handlers = self.stream.stream_handlers_cliente()
		self.core = AppCoreAlumno(self.qtapp, self.gui, self.stream_handlers)
		self.core.set_handlers_appcore(self.appcore_handlers)
		self.gcGrupos = GestionControlGrupos(self.gui, self.core)
		self.gui_handlers = self.gcGrupos.gui_handlers_cliente()
		self.gui.set_core_handlers(self.gui_handlers)

	def setup_interfaz(self):
		self.qtapp = QtGui.QApplication(sys.argv)
		self.qtapp.setOrganizationName("hgr")
		self.qtapp.setApplicationName("Gestion")

	def i10n(self):
		locale = QtCore.QLocale.system().name()
		self.qtTranslator = QtCore.QTranslator()
		if self.qtTranslator.load(self.appgui+locale, ":/tr"):
			#logging.info("Cargando traducciones para: (%s)" % locale)
			self.qtapp.installTranslator(self.qtTranslator)

	def obtener_pid(self):
		sentencia = "ps aux | grep cga-hgr-client | grep -v grep | awk '{print $2}'"
		pid = commands.getstatusoutput(sentencia)
		self.pidCliente= pid[1]

	def cerrar_aplicacion(self):
		self.gui.hide()
		self.qtapp.quit()

		self.obtener_pid()
		if self.pidCliente != "" and self.pidCliente != None:
			os.system('kill -9 '+str(self.pidCliente))
