#!/usr/bin/python
# -*- coding: utf-8 -*-
## \namespace sshareddirclient
import os
import sys
import socket
import urllib

from PyQt4 import QtGui
from PyQt4 import QtCore
import davclient

from hga.controlgrupos.common.settings import SHARED_COMP
from hga.controlgrupos.common.settings import SHARED_SUB
from hga.controlgrupos.common.settings import VALID_CHARS
import hga.gui.common.icon_rc


##\class SSharedDirClient
# \brief Webdav explorer main class
class SSharedDirClient(QtGui.QWidget):
	##\fn __init__(self, parent = None, data = None)
	# \brief Class constructor, create webdav connection and GUI Webdav explorer client
	# \param[in] data: Connection data (ip, port, user, password, nick and group)
	def __init__(self, parent = None, data = None):
		#Creacion de ventana principal compartir
		QtGui.QWidget.__init__(self, parent)

		data['group'] = self.sanitize(data['group'])
		data['nick'] = self.sanitize(data['nick'])

		self.remotedir = os.path.sep + data['group']+ os.path.sep + SHARED_COMP + os.path.sep + data['nick']
		self.remotedir2 = os.path.sep + data['group']+ os.path.sep + SHARED_SUB 
		self.davbackend = clientDAVBackend(data)
		self.setWindowTitle("HGR - Carpeta compartida")
		self.resize(560, 520)
		self.setMinimumSize(560, 520)
		self.upspace = uploadWidget(parent = self, wdb = self.davbackend, remotedir = self.remotedir)
		self.expwebdav = explorerDAV(parent = self, wdb = self.davbackend, remotedir2 = self.remotedir2)
		icon = QtGui.QIcon()
		icon.addPixmap(QtGui.QPixmap(":/common/icon/share.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		self.setWindowIcon(icon)

		#Boton archivos subidos
		vboxmain = QtGui.QVBoxLayout()
		self.setLayout(vboxmain)
		hboxup = QtGui.QHBoxLayout()
		hboxup.addWidget(self.upspace)
		remotelistbutton = QtGui.QPushButton("", self)
		self.connect(remotelistbutton, QtCore.SIGNAL("clicked()"), self.remote_list_file)
		remotelistbutton.setStyleSheet("background-color: rgb(153, 153, 153); background-image: url(:/common/icon/folder.png); background-repeat: no-repeat; background-position: center")
		remotelistbutton.setMaximumSize(100, 100)
		remotelistbutton.setToolTip("Ver archivos subidos")
		hboxup.addWidget(remotelistbutton)

		vboxmain.addLayout(hboxup)
		vboxmain.addWidget(self.expwebdav)

	def remote_list_file(self):
		#Ventana archivos subidos por cliente
		self.listfilewindow = QtGui.QWidget()
		self.listfilewindow.setWindowTitle('Archivos subidos')
		self.listfilewindow.resize(460, 500)
		self.listfilewindow.setMinimumSize(460, 500)
		icon2 = QtGui.QIcon()
		icon2.addPixmap(QtGui.QPixmap(":/common/icon/upload.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		self.listfilewindow.setWindowIcon(icon2)
		vbox = QtGui.QVBoxLayout()
		self.listfilewindow.setLayout(vbox)
		self.filelistupload = explorerDAV(parent = self.listfilewindow, wdb = self.davbackend, userdir = self.remotedir, viewmode = 1)
		vbox.addWidget(self.filelistupload)
		hbox = QtGui.QHBoxLayout()
		upicon = QtGui.QIcon()
		upicon.addPixmap(QtGui.QPixmap(":/common/icon/up2.png"))
		homeicon = QtGui.QIcon()
		homeicon.addPixmap(QtGui.QPixmap(":/common/icon/home.png"))
		upbutton = QtGui.QPushButton(upicon, "", self)
		self.connect(upbutton, QtCore.SIGNAL("clicked()"), self.up_level)
		upbutton.setMinimumSize(32, 32)
		upbutton.setToolTip("Subir de nivel")
		homebutton = QtGui.QPushButton(homeicon, "", self)
		self.connect(homebutton, QtCore.SIGNAL("clicked()"), self.goto_init)
		homebutton.setMinimumSize(32, 32)
		homebutton.setToolTip("Inicio")
		hbox.addWidget(upbutton)
		hbox.addWidget(homebutton)
		vbox.addLayout(hbox)
		self.listfilewindow.show()

	def sanitize(self, data):
		data = data.lstrip()
		data = data.replace(' ', '_')
		for a,b in ( ('á', 'a'), ('é', 'e'), ('í', 'i'), ('ó', 'o'), ('ú', 'u'), ('ñ', 'n'),
					('Á', 'A'), ('É', 'E'), ('Í', 'I'), ('Ó', 'O'), ('Ú', 'U'), ('Ñ', 'N') ):
			data=data.replace(a, b)
		data = str(filter(lambda k: k in VALID_CHARS, data)).lower()
		return data


	##\fn goto_init(self)
	# \brief Go to initial webdav directory
	#Ir al directorio inicial
	def goto_init(self):
		self.filelistupload.current_dir = self.remotedir
		self.filelistupload.level = 6
		self.filelistupload.fill_explorer()

	##\fn up_level(self)
	# \brief Up a one level dir on webdav explorer
	#Subir unn nivel del directorio
	def up_level(self):
		if self.filelistupload.level == 6:
			return
		string = self.filelistupload.current_dir.split('/')
		self.filelistupload.current_dir = ''
		for i in range(1,len(string)-1):
			self.filelistupload.current_dir = self.filelistupload.current_dir + '/' + string[i]
		self.filelistupload.level = self.filelistupload.level - 1
		self.filelistupload.fill_explorer()


##\class uploadWidget
# \brief Upload file widget
class uploadWidget(QtGui.QFrame):
	##\fn __init__(self, parent = None, wdb = None)
	# \brief Create upload widget
	# \param[in] wdb: Webdav connection backend
	# \param[in] remotedir: User remote directory in webdav server
	def __init__(self, parent = None, wdb = None, remotedir = None):
		#Espacio para subir los archivos que se desean compartir
		QtGui.QFrame.__init__(self, parent)
		self.davbackend = wdb
		self.remotedir = remotedir
		self.setMinimumSize(400, 100)
		self.setFrameShape(QtGui.QFrame.StyledPanel)
		self.setFrameShadow(QtGui.QFrame.Raised)
		self.setStyleSheet("background-color: rgb(153, 153, 153);")
		self.setLineWidth(1)
		self.setMidLineWidth(1)
		self.setAcceptDrops(True)
		vbox = QtGui.QVBoxLayout()
		self.setLayout(vbox)
		vbox.setAlignment(QtCore.Qt.AlignHCenter)
		vbox.setAlignment(QtCore.Qt.AlignVCenter)
		label = QtGui.QLabel('Arrastrar y soltar a este espacio\nlos archivos que desea subir')
		label.setStyleSheet("font: bold 14px; color: rgb(255, 255, 255);")
		label.setToolTip("Espacio para subir archivos")
		label.setAlignment(QtCore.Qt.AlignHCenter)
		vbox.addWidget(label)


	##\fn dragEnterEvent(self, event)
	# \brief dragEnterEvent overload
	def dragEnterEvent(self, event):
		event.acceptProposedAction()

	##\fn dropEvent(self, event)
	# \brief dropEvent overload
	def dropEvent(self, event):
		event.accept()
		pathfiles = str(event.mimeData().text()).replace('file://','').split('\n')
		error = False
		
		for pathfile in pathfiles:
			if pathfile == '':
				continue
			if os.path.isdir(pathfile.strip()):
				self.upload_directory(pathfile.strip())
			else:

				try:
					filename = pathfile.split('/')[-1]
					fich = open(urllib.unquote(pathfile.strip()), 'r')
					cont = fich.readline()
					if cont == "":
						QtGui.QMessageBox.warning(self, u'Error', u'<b>No es posible subir archivos sin contenido</b>')
						error = True
						fich.close()
						continue
					fich.seek(0, os.SEEK_SET)
					self.davbackend.upload_file(self.remotedir + os.path.sep + self.sanitize(filename), fich)
					fich.close()
				except IOError, msg:
					QtGui.QMessageBox.warning(self, u'Error', u'<b>El archivo o archivos no han podido ser enviados</b>')
					error = True
		if not error:
			QtGui.QMessageBox.information(self, u'Información', u'<br>Archivos subidos correctamente')

	def sanitize(self, data):
		data = data.replace(' ', '_')
		for a,b in ( ('á', 'a'), ('é', 'e'), ('í', 'i'), ('ó', 'o'), ('ú', 'u'), ('ñ', 'n'),
					('Á', 'A'), ('É', 'E'), ('Í', 'I'), ('Ó', 'O'), ('Ú', 'U'), ('Ñ', 'N') ):
			data=data.replace(a, b)
		return data


	##\fn upload_directory(self, path)
	# \brief Metodo encargado de subir directorios y todo su contenido
	def upload_directory(self, path):
		dir = os.path.sep + path.split('/')[-1]
		self.davbackend.creaDirec(self.remotedir + dir)
		for dirName, subdirList, fileList in os.walk(path):

			for subdir in subdirList:
				auxdir = dir + dirName.split(dir)[-1] + os.path.sep + subdir
				self.davbackend.creaDirec(self.remotedir + auxdir)

			auxdir = dir + dirName.split(dir)[-1]

			for file in fileList:
				fich = open(urllib.unquote(dirName + os.path.sep + file), 'r')
				cont = fich.readline()
				if cont == "":
					fich.close()
					continue
				else:
					fich.seek(0, os.SEEK_SET)
					self.davbackend.upload_file(self.remotedir + auxdir + os.path.sep + self.sanitize(file), fich)
				fich.close()


##\class explorerDAV
# \brief Webdabav file browser
class explorerDAV(QtGui.QWidget):
	#Ventana principal compartir
	##\fn __init__(self, parent = None, wdb = None, userdir = None, viewmode = 0)
	# \brief Create gui explorer
	# \param[in] wdb: WebDav connection backend
	# \param[in] userdir: user remote directory (only for viewmode = 1)
	# \param[in] viewmode: webdav client explorer viewmode, 0 is normal mode, 1 is minimal mode(only user remote file list)
	def __init__(self, parent = None, wdb = None, userdir = None, viewmode = 0, remotedir2 = None):
		QtGui.QWidget.__init__(self, parent)
		self.davbackend = wdb
		self.remotedir2 = remotedir2
		vboxexp = QtGui.QVBoxLayout()
		self.setLayout(vboxexp)

		if viewmode == 0:
			label = QtGui.QLabel("Archivos compartidos", self)
		else:
			label = QtGui.QLabel("Mis archivos subidos", self)
		label.setStyleSheet("font: bold 14px; color: rgb(0, 0, 0);")
		vboxexp.addWidget(label)

		if viewmode == 0:
			hboxbuttonsexp = QtGui.QHBoxLayout()
			vboxexp.addLayout(hboxbuttonsexp)
			hboxbuttonsexp.setAlignment(QtCore.Qt.AlignLeft)
			upicon = QtGui.QIcon()
			upicon.addPixmap(QtGui.QPixmap(":/common/icon/up2.png"))
			reloadicon = QtGui.QIcon()
			reloadicon.addPixmap(QtGui.QPixmap(":/common/icon/refresh.png"))
			homeicon = QtGui.QIcon()
			homeicon.addPixmap(QtGui.QPixmap(":/common/icon/home.png"))
			viewicons = QtGui.QIcon()
			viewicons.addPixmap(QtGui.QPixmap(":/common/icon/list-icon.png"))
			viewlist = QtGui.QIcon()
			viewlist.addPixmap(QtGui.QPixmap(":/common/icon/list.png"))
			upbutton = QtGui.QPushButton(upicon, "", self)
			self.connect(upbutton, QtCore.SIGNAL("clicked()"), self.up_level)
			upbutton.setMinimumSize(32, 32)
			upbutton.setToolTip("Subir de nivel")
			reloadbutton = QtGui.QPushButton(reloadicon, "", self)
			self.connect(reloadbutton, QtCore.SIGNAL("clicked()"), self.reload_dir)
			reloadbutton.setMinimumSize(32, 32)
			reloadbutton.setToolTip("Refrescar")
			homebutton = QtGui.QPushButton(homeicon, "", self)
			self.connect(homebutton, QtCore.SIGNAL("clicked()"), self.goto_init)
			homebutton.setMinimumSize(32, 32)
			homebutton.setToolTip("Inicio")
			viewiconsbutton = QtGui.QPushButton(viewicons, "", self)
			self.connect(viewiconsbutton, QtCore.SIGNAL("clicked()"), self.set_icons_view)
			viewiconsbutton.setMinimumSize(32, 32)
			viewiconsbutton.setToolTip("Ver archivos en modo iconos")
			viewlistbutton = QtGui.QPushButton(viewlist, "", self)
			self.connect(viewlistbutton, QtCore.SIGNAL("clicked()"), self.set_list_view)
			viewlistbutton.setMinimumSize(32, 32)
			viewlistbutton.setToolTip("Ver archivos en modo lista")
			hboxbuttonsexp.addWidget(upbutton)
			hboxbuttonsexp.addWidget(reloadbutton)
			hboxbuttonsexp.addWidget(homebutton)
			hboxbuttonsexp.addWidget(viewiconsbutton)
			hboxbuttonsexp.addWidget(viewlistbutton)

		self.filespace = QtGui.QListWidget()
		self.filespace.setMovement(0)
		self.filespace.setViewportMargins(5, 5, 5, 5)
		if viewmode == 0:
			self.set_icons_view()
		else:
			self.set_list_view()
		self.filespace.setSortingEnabled(True)
		self.connect(self.filespace, QtCore.SIGNAL("itemActivated(QListWidgetItem *)"), self.enter_dir)
		vboxexp.addWidget(self.filespace)

		if viewmode == 0:
			hboxbuttons = QtGui.QHBoxLayout()
			downloadicon = QtGui.QIcon()
			downloadicon.addPixmap(QtGui.QPixmap(":/common/icon/download.png"))
			downloadbutton = QtGui.QPushButton(downloadicon, "Descargar archivo", self)
			downloadbutton.setStyleSheet("font: bold 14px; color: rgb(0, 0, 0);")
			self.connect(downloadbutton, QtCore.SIGNAL("clicked()"), self.download_target_selection)
			downloadbutton.setMaximumSize(180, 64)
			hboxbuttons.addWidget(downloadbutton)
			vboxexp.addLayout(hboxbuttons)

		resp = self.davbackend.dir_access('/')
		if resp == '-1':
			print "No hay que ejecutar la aplicacion"
			QtGui.QMessageBox.warning(self, u"Atención", u"<br>Servicio de archivos compartidos desactivado. Vuelve a intentarlo.", u"Cerrar")
		else:
			if viewmode == 0:
				self.current_dir = self.remotedir2
				self.level = 5
				self.fill_explorer()
			else:
				self.current_dir = userdir
				self.level = 6
				self.fill_explorer()

	##\fn fill_explorer(self)
	# \fn Load webdav directory based in self.current_dir and self.level
	def fill_explorer(self):
		self.filespace.clear()
		resp = self.davbackend.dir_access(self.current_dir)
		if resp == '-1':
			QtGui.QMessageBox.warning(self, u"Atención", u"<br>Servicio de archivos compartidos desactivado. Vuelve a intentarlo.", u"Cerrar")
			return '-1'
		if resp != None:
			for elem in resp.keys():
				string = elem.split('/')[self.level:]
				try:
					if string[0] == SHARED_COMP:
						continue
					icon = QtGui.QIcon()
					if resp[elem]['getcontenttype'] == 'httpd/unix-directory' and string[0]:
						icon.addPixmap(QtGui.QPixmap(":/common/icon/folder-2.png"))
						item_type = 0
					else:
						icon=self.mime_type(string[0])
						item_type = 1
					item = explorerItem(icon, unicode(urllib.unquote(string[0]), 'utf-8').encode('latin-1'))
					item.set_type(item_type)
					self.filespace.addItem(item)

				except IndexError, msg:
					continue
				except KeyError, msg:
					# File without valid extension: Check if it is a temporal copy (ends with "~" === "%7E"), and if it isn't we display it
					if string[0][-3:] != '%7E':
						icon=self.mime_type(string[0])
						item_type = 1
						item = explorerItem(icon, unicode(urllib.unquote(string[0]), 'utf-8').encode('latin-1'))
						item.set_type(item_type)
						self.filespace.addItem(item)
					continue

	##\fn reload_dir(self)
	# \brief Reload current directory
	#Actualizar directorio actual
	def reload_dir(self):
		self.fill_explorer()

	##\fn goto_init(self)
	# \brief Go to initial webdav directory
	#Ir al directorio inicial
	def goto_init(self):
		self.current_dir = self.remotedir2
		self.level = 5
		self.fill_explorer()

	##\fn up_level(self)
	# \brief Up a one level dir on webdav explorer
	#Subir unn nivel del directorio
	def up_level(self):
		if self.level == 5:
			return
		string = self.current_dir.split('/')
		self.current_dir = ''
		for i in range(1,len(string)-1):
			self.current_dir = self.current_dir + '/' + string[i]
		self.level = self.level - 1
		self.fill_explorer()


	##\fn enter_dir(self)
	# \brief Access to directory selected
	#Acceder al directorio seleccionado
	def enter_dir(self):
		selected_item = self.filespace.currentItem()
		if selected_item is not None:
			row = self.filespace.row(selected_item)
			item = self.filespace.item(row)
			if item.item_type == 1:
				return False
			else:
				self.level = self.level + 1
				self.current_dir = self.current_dir + os.path.sep + item.text()
				self.fill_explorer()

	##\fn download_target_selection(self)
	# \brief Download file selected in explorer
	#Descargar el archivo seleccionado
	def download_target_selection(self):
		selected_item = self.filespace.currentItem()
		if selected_item is not None:
			row = self.filespace.row(selected_item)
			item = self.filespace.item(row)
			if item.item_type == 0:
				return False
			else:
				name = self.current_dir + os.path.sep + item.text()
				name.replace(' ','%20')
				target = os.environ.get('HOME') + os.path.sep + "Descargas" +os.path.sep + item.text()
				filename = QtGui.QFileDialog.getSaveFileName(self, 'Guardar Como', target, 'All files (*)')
				try:
					if filename != '':
						fich = open(filename, 'w')
						fich.write(self.davbackend.download_file(name))
						QtGui.QMessageBox.information(self, u"Información", u"<br>Archivo descargado con éxito.")
						fich.close()
				except socket.error, msg:
					fich.close()
					QtGui.QMessageBox.warning(self, u"Error", u"""Descarga fallida. <br> Servicio de archivos compartidos desactivado o no estás conectado a ningún grupo. Vuelve a intentarlo. """)
					os.remove(filename)

	def mime_type(self, name):
		icon = QtGui.QIcon()
		ext = name.split('.')[-1]
		if ext == 'html' or ext == 'htm':
			return icon.fromTheme("libreoffice-oasis-web-template")
		elif ext == 'png' or ext == 'jpg' or ext == 'jpeg' or ext == 'gif' or ext == 'bmp':
			return icon.fromTheme("image-x-generic")
		elif ext == 'mp3' or ext == 'ogg' or ext == 'wma' or ext == 'mp4' or ext == 'cda' or ext == 'aac' or ext == 'wav':
			return icon.fromTheme("audio-x-adpcm")
		elif ext == 'ods':
			return icon.fromTheme("libreoffice-oasis-spreadsheet")
		elif ext == 'odp':
			return icon.fromTheme("libreoffice-oasis-presentation")
		elif ext == 'odt':
			return icon.fromTheme("libreoffice-oasis-text")
		elif ext == 'pdf':
			return icon.fromTheme("application-pdf")
		elif ext == 'odb':
			return icon.fromTheme("libreoffice-oasis-database")
		elif ext == 'doc':
			return icon.fromTheme("application-msword")
		elif ext == 'txt':
			return icon.fromTheme("libreoffice-oasis-master-document")
		elif ext == 'odg':
			return icon.fromTheme("libreoffice-oasis-drawing")
		elif ext == 'avi' or ext == 'mpg' or ext == 'mpeg' or ext == 'mov' or ext == 'flv' or ext == 'wmv' or ext == 'mkv':
			return icon.fromTheme("video-x-generic")
		elif ext == 'zip' or ext == 'gz' or ext == 'bz' or ext == 'bz2' or ext == 'rar' or ext == 'deb':
			return icon.fromTheme("application-x-ace")
		else:
			return icon.fromTheme("text-x-preview")

	def set_icons_view(self):
		self.filespace.setViewMode(1)
		self.filespace.setGridSize(QtCore.QSize(120, 58))

	def set_list_view(self):
		self.filespace.setViewMode(0)
		self.filespace.setGridSize(QtCore.QSize(120, 30))

##\class explorer_element
# \brief
class explorerItem(QtGui.QListWidgetItem):
	def set_type(self, item_type):
		self.item_type = item_type

##\class clientDAVBackend
# \brief Manage the webdav client connection
class clientDAVBackend:
	##\fn __init__(self, data, remotedir)
	# \brief Configure connection with parameters IP, port, user and password
	# \param[in] data: client data configuration
	def __init__(self, data):
		self.connection = davclient.DAVClient('http://%s:%s' % (data['IP'], data['port']))
		self.connection.set_basic_auth(data['user'], data['password'])
		remotegroupdir = os.path.sep + data['group']+ os.path.sep + SHARED_COMP 
		remoteuserdir = remotegroupdir + os.path.sep + data['nick']
		self.connection.mkcol(remotegroupdir)
		self.connection.mkcol(remoteuserdir)

	##\fn dir_access(self, dir)
	# \brief Access to remote webdav directory
	# \param[in] dir: directory to access
	# \param[out] Directory data accessed
	def dir_access(self, dir):
		try:
			return self.connection.propfind(dir, depth = 1)
		except:
			print "Error de conexion con servidor WebDav"
			return '-1'

	##\fn download_file(self, path)
	# \brief Download file in webdav server path
	# \param[in] path: path to file to download
	def download_file(self, path):
		return self.connection.get(path)

	def upload_file(self, name, data):
		self.connection.put(name, f = data)

	def creaDirec(self, path):
		self.connection.mkcol(path)

#app = QtGui.QApplication(sys.argv)
#datos = {"IP": "10.227.255.167", "port": "8008", "user": "usuario", "password": "usuario" }
#wdclient = SSharedDirClient(data = datos)
#wdclient.show()

#sys.exit(app.exec_())
