#!/usr/bin/python
# -*- coding: utf-8 -*-
## \namespace controldesktop
import os
import commands
from subprocess import Popen, PIPE
from socket import socket, error
import time
import random
import logging
from PyQt4 import QtCore
from hga.controlgrupos.common.user_info import get_ip
from hga.controlgrupos.common.user_info import ping
from hga.controldesktop.settings import PASS_DIR
from hga.controldesktop.settings import genera_clave
from hga.controldesktop.settings import guardar_clave
from hga.controldesktop.controlpeticiones import finalizar_proceso_x11vnc
from hga.controldesktop.controlpeticiones import servicio_vnc_activo
from hga.controldesktop.controlpeticiones import servicio_demo_activo
from hga.controlgrupos.common.exception import NoDefaultDevice
from hga.controlgrupos.common.settings import LOGGING_CONFIG_FILE

logging.config.fileConfig(LOGGING_CONFIG_FILE)
lerror = logging.error

## \class ControlDesktop()
#  \brief Clase que administra las peticiones del servidor de control sobre el cliente.
class ControlDesktop():
	def __init__(self):
		self.core = None
		self.ipServidor = None
		self.clave = None
		self.ip_user_emision = None

	## \fn def apagar_equipo_remoto(self, core, nick=None):
	# \brief Envía mensaje al cliente para ejecutar el apagado del equipo.
	#  \param core Objeto core de la aplicación
	#  \param nick Nick del usuario en caso de que se solicite una acción individual.
	def apagar_equipo_remoto(self, core, nick=None):
		self.core = core
		msg = "DESKTOP|APAGAR"
		if not nick:
			self.enviar_mensaje_grupo(msg)
		else:
			self.core.sendmessage(nick,msg)

	## \fn def reiniciar_equipo_remoto(self, core, nick=None):
	# \brief Envía mensaje al cliente para ejecutar el reinicio del equipo.
	#  \param core Objeto core de la aplicación
	#  \param nick Nick del usuario en caso de que se solicite una acción individual.
	def reiniciar_equipo_remoto(self, core, nick=None):
		self.core = core
		msg = "DESKTOP|REINICIAR"
		if not nick:
			self.enviar_mensaje_grupo(msg)
		else:
			self.core.sendmessage(nick,msg)

	## \fn def bloquear_pantalla_remota(self, core, nick=None):
	# \brief Envía mensaje al cliente para ejecutar el bloqueo del equipo.
	#  \param core Objeto core de la aplicación
	#  \param nick Nick del usuario en caso de que se solicite una acción individual.
	def bloquear_pantalla_remota(self, core, nick=None):
		self.core = core
		msg = "DESKTOP|LOCK"
		if not nick:
			self.enviar_mensaje_grupo(msg)
		else:
			self.core.sendmessage(nick,msg)

	## \fn def desbloquear_pantalla_remota(self, core, nick=None):
	# \brief Envía mensaje al cliente para ejecutar el desbloqueo del equipo.
	#  \param core Objeto core de la aplicación
	#  \param nick Nick del usuario en caso de que se solicite una acción individual.
	def desbloquear_pantalla_remota(self, core, nick=None):
		self.core = core
		msg = "DESKTOP|UNLOCK"
		if not nick:
			self.enviar_mensaje_grupo(msg)
		else:
			self.core.sendmessage(nick,msg)

	## \fn def buscar_aplicacion(self, core, app, nick=None):
	# \brief Busca las aplicaciones en /usr/share/applications/*.desktop cuyos nombres se correspondan con el patrón indicado por el usuario.
	#  \param core Objeto core de la aplicación
	#  \param app Mensaje introducido por el profesor.
	#  \param nick Nick del usuario. Si se solicta el envío para el grupo completo, será None.
	def buscar_aplicacion(self, core, app, nick=None):
		self.core = core
		buscar_desktop="grep -e '^Name=' /usr/share/applications/*.desktop | grep -i '%s' | cut -d '/' -f 5 | cut -d ':' -f 1 | head -1" % app
		desktop = commands.getoutput(buscar_desktop)
		ejecutable = "/usr/share/applications/%s" %desktop
		msg = "DESKTOP|APP|"+ejecutable
		if not nick:
			self.enviar_mensaje_grupo(msg)
		else:
			self.core.sendmessage(nick,msg)

	## \fn def ejecutar_navegador(self, core, url, nick=None):
	# \brief Funcion que se encarga de enviar el mensaje para la ejecución de Firefox en cliente.
	#  \param url URL indicada por el profesor con la que se abrirá el navegador.
	#  \param nick Nick del usuario. Si se solicta el envío para el grupo completo, será None.
	def ejecutar_navegador(self, core, url, nick=None):
		self.core = core
		msg = "DESKTOP|NAVEGADOR|"+url
		if not nick:
			self.enviar_mensaje_grupo(msg)
		else:
			self.core.sendmessage(nick,msg)

	## \fn def bloquear_internet(self, core, nick=None):
	# \brief Funcion que se encarga de enviar el mensaje para bloquear internet en el cliente.
	#  \param core Objeto core de la aplicación
	#  \param nick Nick del usuario. Si se solicta el envío para el grupo completo, será None.
	def bloquear_internet(self, core, nick=None):
		self.core = core
		msg = "DESKTOP|INTERNET_BLOCK"
		if not nick:
			self.enviar_mensaje_grupo(msg)
		else:
			self.core.sendmessage(nick,msg)

	## \fn def desbloquear_internet(self, core, nick=None):
	# \brief Funcion que se encarga de enviar el mensaje para desbloquear internet en el cliente.
	#  \param core Objeto core de la aplicación
	#  \param nick Nick del usuario. Si se solicta el envío para el grupo completo, será None.
	def desbloquear_internet(self, core, nick=None):
		self.core = core
		msg = "DESKTOP|INTERNET_UNBLOCK"
		if not nick:
			self.enviar_mensaje_grupo(msg)
		else:
			self.core.sendmessage(nick,msg)

	## \fn def abrir_open_office(self, core, nick=None):
	# \brief Funcion que se encarga de enviar el mensaje para la ejecución en el cliente de LibreOffice Writer
	#  \param core Objeto core de la aplicación
	#  \param nick Nick del usuario. Si se solicta el envío para el grupo completo, será None.
	def abrir_open_office(self, core, nick=None):
		self.core = core
		msg = "DESKTOP|OFFICE"
		if not nick:
			self.enviar_mensaje_grupo(msg)
		else:
			self.core.sendmessage(nick,msg)

	## \fn def enviar_mensaje_info(self, core, msg, nick=None):
	# \brief Funcion que se encarga de enviar el mensaje informativo que desee el profesor a los alumnos.
	#  \param core Objeto core de la aplicación
	#  \param tipo_acceso Mensaje introducido por el profesor.
	#  \param nick Nick del usuario. Si se solicta el envío para el grupo completo, será None.
	def enviar_mensaje_info(self, core, msg, nick=None):
		self.core = core
		msg = "DESKTOP|INFO|"+msg
		if not nick:
			self.enviar_mensaje_grupo(msg)
		else:
			self.core.sendmessage(nick,msg)

	## \fn def demostracion_grupo(self, core, tipo_acceso):
	# \brief Funcion que realiza la demostración al grupo completo
	#  \param core Objeto core de la aplicación
	#  \param tipo_acceso Tipo de acción que se solicita o se desea realizar (DEMO_GRUPO)
	def demostracion_grupo(self, core, tipo_acceso):
		try:
			ipServidor = get_ip()
			servicio_activo = False
			self.core = core
		
			for user in self.core.users:
				ipUsuario = self.core.users[user].get("IP")
				if servicio_vnc_activo(ipUsuario): # Se comprueba si hay algún servicio VNC activo.
					servicio_activo = True

			if not servicio_activo:
				if ping(ipServidor):
					self.clave = genera_clave()
					guardar_clave(self.clave)
					os.spawnlp(os.P_NOWAIT, 'x11vnc', 'x11vnc', '-nonc','-viewonly','-rfbauth', '-shared', '-many', PASS_DIR, '-rfbport', '5905') 
					msg = "DESKTOP|DEMO"+"|"+self.clave+"|"+ipServidor
					time.sleep(2)
					self.enviar_mensaje_grupo(msg)
			else:
				# Indicamos a la GUI que no se ha podido activar el proceso
				self.core.enviar_gui_msg(u"Hay algún servicio de acceso remoto activo. Por favor, finalice antes de realizar una demostración al grupo", "information")
				self.core.stop_progressBar()
				self.core.gui.emit(QtCore.SIGNAL('estadoDemoGrupo'),False)
		except NoDefaultDevice, e:
				# Indicamos a la GUI que no se ha podido activar el proceso
				self.core.stop_progressBar()
				self.core.gui.emit(QtCore.SIGNAL('estadoDemoGrupo'),False)
				lerror("Petición -> Realizar demostración. No hay conexión: %s" % e)
				self.core.enviar_gui_msg(u"No se puede realizar la demostración en este momento.<br>Se ha perdido la conexión.", "error")

	## \fn def demostracion_usuario(self, nick, core, tipo_acceso):
	# \brief Funcion que realiza la demostración a un usuario
	#  \param nick Es el nick del usuario al que se le envia el mensaje para que se conecte al servidor.
	#  \param core Objeto core de la aplicación
	#  \param tipo_acceso Tipo de acción que se solicita o se desea realizar (DEMO_USER)
	def demostracion_usuario(self, nick, core, tipo_acceso):
		try:
			ipServidor = get_ip() # IP del profesor, que lanzará el servidor
			self.core = core
			ipUsuario = self.core.users[nick].get("IP") # IP del usuario que se conectará al servidor para visualizar la demostración.
			if self.configurar_servicio(ipUsuario):
				if ping(ipServidor):
					os.spawnlp(os.P_NOWAIT, 'x11vnc', 'x11vnc', '-nonc', '-viewonly','-rfbauth', '-shared', '-many', PASS_DIR, '-rfbport', '5905') 
					msg = "DESKTOP|DEMO"+"|"+self.clave+"|"+ipServidor
					time.sleep(2)
					self.core.sendmessage(nick,msg)
			else:
				# Indicamos a la GUI que no se ha podido activar el proceso
				self.core.enviar_gui_msg(u"Finalice el acceso al equipo de %s antes de realizar una demostración." % nick, "information")
				self.core.stop_progressBar()
				self.core.gui.emit(QtCore.SIGNAL('estadoDemo'),False)
		except NoDefaultDevice, e:
			self.core.stop_progressBar()
			self.core.gui.emit(QtCore.SIGNAL('estadoDemo'),False)
			lerror("Petición -> Realizar demostración. No hay conexión: %s" % e)
			self.core.enviar_gui_msg(u"No se puede realizar la demostración en este momento.<br>Se ha perdido la conexión.", "error")


	## \fn def stop_demostracion(self):
	# \brief Funcion que solicita la finalización del proceso vnc cuando realiza la demostración a los usuarios.
	def stop_demostracion(self, core, nick=None):
		if not nick:
			finalizar_proceso_x11vnc() # Se para el servidor vnc, finalizando por tanto el acceso de los clientes.
		else:
			msg="DESKTOP|STOP_DEMO"
			self.core.sendmessage(nick,msg) # Se envía al cliente un mensaje para que finalice el acceso al servidor.
		self.core.gui.emit(QtCore.SIGNAL('estadoDemoGrupo'),False)

	## \fn def solicitar_acceso_remoto(self, nick, core, tipo_acceso):
	# \brief Funcion para solicitar el acceso remoto a un alumno, para visualizar y/o controlar
	#  \param nick Es el nick del usuario al que se le envia el mensaje solicitando la activación del servidor VNC.
	#  \param core Objeto core de la aplicación
	#  \param tipo_acceso Tipo de acción que se solicita o se desea realizar (controlar o sólo visualizar)
	def solicitar_acceso_remoto(self, nick, core, tipo_acceso):
		self.core = core
		msg = ""
		self.ipServidor = self.core.users[nick].get("IP")
		if self.configurar_servicio(self.ipServidor):
			if self.ip_user_emision == None:
				if ping(self.ipServidor):
					msg = "DESKTOP|"+tipo_acceso+"|"+self.clave
					self.core.sendmessage(nick, msg)
			else:
				self.core.stop_progressBar()
		else:
			self.core.stop_progressBar()
			self.core.enviar_gui_msg(u"Debe finalizar el acceso remoto sobre %s, o finalizar la demostración al grupo antes de ejecutar esta funcionalidad." % nick, "information")

	## \fn def acceso_remoto(self, nick, tipo_acceso):
	# \brief Función que genera el comando para acceder a un equipo remoto y lo envía mediante mensaje para lanzar la petición.
	def acceso_remoto(self):
		self.core.stop_progressBar()
		pid_vnc = os.spawnlp(os.P_NOWAIT, 'xvnc4viewer', 'xvnc4viewer', '-PasswordFile', PASS_DIR, str(self.ipServidor)+"::5905")

	## \fn def solicitar_emision_alumno_grupo(self, nick, core, tipo_acceso):
	# \brief Funcion que realiza la petición de demostración de un alumno al resto del grupo, sin incluir al profesor, o gestor de grupo.
	#  \param nick Es el nick del usuario al que se le envia el mensaje para que levante el servicio.
	#  \param core Objeto core de la aplicación
	#  \param tipo_acceso Tipo de acción que se solicita o se desea realizar
	def solicitar_emision_alumno_grupo(self, nick, core, tipo_acceso):
		try:
			servicio_activo = False
			self.core = core
			msg = ""
			self.ip_user_emision = self.core.users[nick].get("IP")

			for user in self.core.users: 
				ipUsuario = self.core.users[user].get("IP")
				# Se comprueba si hay algun servicio VNC activo para cualquiera de los alumnos
				if not self.configurar_servicio(ipUsuario):
					servicio_activo = True

			if not servicio_activo:
				if ping(self.ip_user_emision): #Se comprueba conexión con el alumno que realizará la emisión
					self.clave = genera_clave()
					guardar_clave(self.clave)

					msg = "DESKTOP|"+tipo_acceso+"|"+self.clave
					self.core.sendmessage(nick, msg)
			else:
				self.ip_user_emision = None
				self.core.gui.emit(QtCore.SIGNAL('estadoEmisionAluGrupo'),False)
				self.core.enviar_gui_msg(u"Hay algún servicio de acceso remoto activo. Por favor, finalice antes de solicitar esta funcionalidad.", "information")
		except NoDefaultDevice, e:
			self.ip_user_emision = None
			self.core.gui.emit(QtCore.SIGNAL('estadoEmisionAluGrupo'),False)
			lerror("Petición -> Solicitar emisión del usuario al grupo. No hay conexión: %s" % e)
			self.core.enviar_gui_msg(u"No se puede lanzar la emisión en este momento.<br>Se ha perdido la conexión.", "error")

	## \fn def stop_emision_alumno_grupo(self, nick, core, tipo_acceso):
	# \brief Solicita la parada de los servicios VNC en cliente (funcionalidad Emisión del alumno al grupo)
	#  \param nick Es el nick del usuario servidor (x11vnc)
	#  \param core Objeto core de la aplicación que se usará para obtener la lista de usuarios
	def stop_emision_alumno_grupo(self, nick, core):
		self.core = core

		if self.ip_user_emision != None and self.ip_user_emision == self.core.users[nick].get("IP"): #Independientemente del control que se realice en la GUI, nos
			# aseguramos de que la ip de la emisión coincide con la IP del usuario seleccionado
			for user in self.core.users:
				if self.ip_user_emision != self.core.users[user].get("IP"): # Alumnos clientes (xvnc4viewer)
					alumno = self.core.users[user].get("nick")
					msg="DESKTOP|STOP_DEMO"
					self.core.sendmessage(alumno,msg)
				else:
					msg="DESKTOP|STOP_DEMO_SERVER"
					self.core.sendmessage(nick,msg) # Alumno servidor (x11vnc)

			self.clave = None
			self.ip_user_emision = None

	## \fn def ver_emision_alumno_grupo(self, nick, core, tipo_acceso):
	# \brief Funcion que realiza la petición para que el grupo visualice la demostración del alumno que ha levantado el servicio.
	def ver_emision_alumno_grupo(self):
		try:
			if self.ip_user_emision != None:
				if ping(self.ip_user_emision): # Se vuelve a comprobar la conexión con el usuario servidor
					msg = "DESKTOP|VER_DEMO_ALU"+"|"+self.clave+"|"+self.ip_user_emision
					time.sleep(2)
					self.enviar_mensaje_grupo(msg)
			else:
				lerror("Petición -> Ver emisión. IP usuario servidor = %s." % self.ip_user_emision)

		except NoDefaultDevice, e:
			self.core.gui.emit(QtCore.SIGNAL('estadoEmisionAluGrupo'),False)
			lerror("Petición -> Realizar demostración. No hay conexión: %s" % e)
			self.core.enviar_gui_msg(u"No se puede realizar la demostración en este momento.<br>Se ha perdido la conexión.", "error")

	## \fn def configurar_servicio(self, nick, tipo_acceso):
	# \brief Solicita información para saber si el servicio VNC está o no activo.
	#  \param ipServidor IP. No es necesario según el caso (ejemplo: demostración al grupo)
	def configurar_servicio(self, ip):
		if not servicio_vnc_activo(ip): # Si no hay servicio activo
			self.clave = genera_clave()
			guardar_clave(self.clave)
			return True # Se puede lanzar la petición
		else:
			return False # Se impide lanzar la petición


	## \fn def enviar_mensaje_grupo(self, msg):
	# \brief Función genérica que realiza el envío de mensajes a todos los usuarios conectados cuando se solicita.
	# \param msg Recibe el mensaje correspondiente a la acción que se solicita ejecutar.
	def enviar_mensaje_grupo(self, msg):
		ipAlumno = ""
		if msg.startswith("DESKTOP"):
			mensaje = msg.split("|")
			if mensaje[1]=='VER_DEMO_ALU': #Obtenemos la IP del alumno servidor si la funcionalidad solicitada es Emisión/Demostración de un alumno al resto del grupo.
				ipAlumno = mensaje[3]

		for user in self.core.users:
			if ipAlumno != self.core.users[user].get("IP"): # Envia el mensaje a todos los alumnos menos al alumno servidor si la funcionalidad es Emisión/Demostración de un alumno al resto del grupo.
				nick = self.core.users[user].get("nick")
				self.core.sendmessage(nick,msg)

	## \fn def controldesktop_handler(self):
	# \brief Definición de los handlers que se necesiten.
	def controldesktop_handler(self):
		self.vnc_handlers = {
			"ACCESO_REMOTO": self.acceso_remoto,
			"EMI_ALUGRUPO_ON": self.ver_emision_alumno_grupo,
			"RECIBE_IMAGEN_ESCRITORIO": self.recibe_imagen_escritorio,
		}
		return self.vnc_handlers


	## \fn escritorio_alumno(self, nick)
	# \brief Metodo que crea el mensaje para pedir la imagen del escritorio del alumno(boton panel usuario).
	# se crea el socket para el envio, se abre y se queda a la espera
	# \param nick Nick del usuario al que se le desea hacer la imagen.
	# \param core Objeto core de la aplicación
	def escritorio_alumno(self, core, nick):
		self.core = core
		self.sServidor = socket()
		CONEXION = (get_ip(), 6030)
		# Escuchar peticiones en el puerto 6030.
		self.sServidor.bind(CONEXION)
		self.sServidor.listen(5)

		msg = "ESCRITORIO|"+nick+"|"+get_ip()
		self.core.sendmessage(nick,msg)


	## \fn recibe_imagen_escritorio(self)
	# \brief Metodo que esta a la espera de recibir el mensaje del cliente para volcar la imagen a traves del socket.
	def recibe_imagen_escritorio(self):    
		sck, addr = self.sServidor.accept()
		secuencia = random.sample('ABCDEFGHIJKLMNOPQRSTUVWXYZ', 8)
		fichero = '/tmp/capt'+str(secuencia[0])
		if os.path.exists(fichero):
			fichero = '/tmp/capt'+str(secuencia[1])
		while True:
			# Recibimos la longitud que envia el cliente
			try:
				recibido = sck.recv(1024)
				if recibido:
					f = open(fichero,'wb')
					while recibido:
						f.write(recibido)
						recibido = sck.recv(1024)
                        
					f.close() 
					sck.close()
			except error:
				break
    
		f.close()
		self.sServidor.close()
		self.core.gui.emit(QtCore.SIGNAL('dialogoEscritorio'), fichero)