#!/usr/bin/python
# -*- coding: utf-8 -*-
## \namespace controldesktop
import os
import commands
from subprocess import Popen, PIPE
import threading
import time
import logging

from hga.controldesktop.settings import FINALIZAR_PRO_X11VNC
from hga.controldesktop.settings import FINALIZAR_PRO_XVNC4VIEWER
from hga.controldesktop.settings import SCRIPT_LOCKSCREEN
from hga.controldesktop.settings import guardar_clave
from hga.controldesktop.settings import PASS_DIR
from hga.controldesktop.settings import LANZA_NAVEGADOR
from hga.controldesktop.settings import LANZA_OFFICE
from hga.controldesktop.settings import SCRIPT_SHUTDOWN
from hga.controldesktop.settings import SCRIPT_BLOCKINTERNET
from hga.controldesktop.settings import SCRIPT_UNBLOCKINTERNET
from hga.controlgrupos.common.user_info import get_ip
from hga.controlgrupos.common.exception import NoDefaultDevice
from hga.controlgrupos.common.settings import LOGGING_CONFIG_FILE

logging.config.fileConfig(LOGGING_CONFIG_FILE)
lerror = logging.error

##\clase ControlProcesos
# \informe Control de ejecución de procesos solicitados
class ControlProcesos():
	def __init__(self):
		self.pid_lockscreen = None #PID del proceso que se necesite ejecutar si lo requiere alguna petición
		self.ipServidor = None
		self.stderr = None

	## \fn def ejecutar_peticion(self, user_msg, nick=None):
	# \brief Comprueba la petición que le llega y ejecuta lo necesario para el funcionamiento.
	# \param user_msg Mensaje con el tipo de petición y los argumentos necesarios según el tipo.
	# \param nick Nick del usuario. Puede ser requerido para el envio de mensajes.
	def ejecutar_peticion(self, user_msg, nick=None):
		if user_msg[1] == "CONTROL":
			guardar_clave(user_msg[2])
			os.spawnlp(os.P_NOWAIT, 'x11vnc', 'x11vnc', '-nonc','-rfbauth', PASS_DIR, '-rfbport', '5905')
			time.sleep(2)
			mensaje = "DESKTOP|VNC"
			return mensaje
			
		elif user_msg[1] == "VISUALIZAR":
			guardar_clave(user_msg[2])
			os.spawnlp(os.P_NOWAIT, 'x11vnc', 'x11vnc', '-nonc','-viewonly', '-rfbauth', PASS_DIR, '-rfbport', '5905')
			time.sleep(2)
			mensaje = "DESKTOP|VNC"
			return mensaje

		elif user_msg[1] == "EMISION_ALUGRUPO": #El profesor ha solicitado que el alumno realice una demostración al grupo.
			ip = get_ip()
			
			if not servicio_vnc_activo(ip):#Se levanta el servicio solo en caso de que no lo esté ya, por ejemplo si el servidor/admin
				# ha solicitado control o visualización.
				guardar_clave(user_msg[2])
				os.spawnlp(os.P_NOWAIT, 'x11vnc', 'x11vnc', '-nonc','-viewonly', '-rfbauth', '-shared', '-many', PASS_DIR, '-rfbport', '5905')
				time.sleep(2)

			mensaje = "DESKTOP|EMISION_ALUGRUPO_ON"
			
			return mensaje

		elif user_msg[1] == "VER_DEMO_ALU": #El alumno visualizará la demostración que está realizando otro alumno del grupo.
			guardar_clave(user_msg[2])
			self.ipServidor = user_msg[3]
			os.spawnlp(os.P_NOWAIT, 'xvnc4viewer', 'xvnc4viewer', '-PasswordFile', PASS_DIR, '-FullScreen', '-MenuKey=""', '-UseLocalCursor=0', str(self.ipServidor)+"::5905")
			time.sleep(2)
			return None

		elif user_msg[1] == "DEMO":
			guardar_clave(user_msg[2])
			self.ipServidor = user_msg[3]
			os.spawnlp(os.P_NOWAIT, 'xvnc4viewer', 'xvnc4viewer', '-PasswordFile', PASS_DIR, '-FullScreen', '-MenuKey=""', '-UseLocalCursor=0', str(self.ipServidor)+"::5905")
			time.sleep(2)
			return None

		elif user_msg[1] == "STOP_DEMO": # Finaliza la visualización de escritorio remoto.
			finalizar_proceso_xvnc4viewer()

		elif user_msg[1] == "STOP_DEMO_SERVER": # Finaliza el servicio de acceso.
			finalizar_proceso_x11vnc()

		elif user_msg[1] == "APAGAR": # Apagar el equipo.
			if self.pid_lockscreen == None:
				ejecutar_script(SCRIPT_SHUTDOWN, "-h")
			else:
				try:
					ip = get_ip()
					parar_procesos_activos(self.pid_lockscreen)
					ejecutar_script(SCRIPT_SHUTDOWN, "-h")
				except NoDefaultDevice, e:
					lerror("Petición -> Apagar equipo. No hay conexión y no se pueden realizar algunas funciones necesarias: %s" % e)
			return None

		elif user_msg[1] == "REINICIAR": # Reiniciar el equipo.
			if self.pid_lockscreen == None:
				ejecutar_script(SCRIPT_SHUTDOWN, "-r")
			else:
				try:
					ip = get_ip()
					parar_procesos_activos(self.pid_lockscreen)
					ejecutar_script(SCRIPT_SHUTDOWN, "-r")
				except NoDefaultDevice, e:
					lerror("Petición -> Reiniciar máquina. No hay conexión y no se pueden realizar algunas funciones necesarias: %s" % e)
			return None

		elif user_msg[1] == "INTERNET_BLOCK":
			bloquearInternet_function('block')
			return None

		elif user_msg[1] == "INTERNET_UNBLOCK":
			bloquearInternet_function('unblock')
			return None

		elif user_msg[1] == "NAVEGADOR":
			url = user_msg[2]
			lanzar_peticion(LANZA_NAVEGADOR+url)
			return None

		elif user_msg[1] == "APP": # Ejecutar una aplicación del sistema.
			obtener_ejecutable = "grep '^Exec=' "+user_msg[2]+" | cut -d '=' -f 2 | cut -d '%' -f 1 | head -1"
			ejecutable = commands.getoutput(obtener_ejecutable)
			lanzar_peticion(ejecutable)
			return None

		elif user_msg[1] == "OFFICE": # Ejecutar libre office
			lanzar_peticion(LANZA_OFFICE)
			return None

		elif user_msg[1] == "INFO":
			mensaje = "CLIENT|"+user_msg[2]+"|information-server"
			return mensaje

		elif user_msg[1] == "LOCK": # BLoquear el equipo
			if self.pid_lockscreen == None:
				self.pid_lockscreen = lockscreen_function("lock_screen")
			return None

		elif user_msg[1] == "UNLOCK":
			if self.pid_lockscreen != None: # Desbloquear el equipo
				lockscreen_function("unlock_screen", self.pid_lockscreen)
				self.pid_lockscreen = None
			return None

## \fn def lockscreen_function(accion, lockscreen_pid=None):
# \brief Gestiona el bloqueo/desbloqueo de los equipos
# \param accion Indica la peticion que se envia, bloquear o desbloquear
# \param lockscreen_pid PID del proceso de bloqueo de pantalla(equipo). Necesario para desbloquear.
def lockscreen_function(accion, lockscreen_pid=None):
	if accion == 'lock_screen':
		lockscreen_pid = os.spawnlp(os.P_NOWAIT, SCRIPT_LOCKSCREEN, SCRIPT_LOCKSCREEN, accion)
		return lockscreen_pid
	elif accion == 'unlock_screen':
		finalizar_proceso(lockscreen_pid)
		return lockscreen_pid

## \fn def bloquearInternet_function(accion):
# \brief Gestiona el bloqueo/desbloqueo de internet
# \param accion Indica la peticion que se envia, bloquear o desbloquear
def bloquearInternet_function(accion):
	if accion == 'block':
		os.spawnlp(os.P_NOWAIT, 'sudo', 'sudo', SCRIPT_BLOCKINTERNET)
	elif accion == 'unblock':
		os.spawnlp(os.P_NOWAIT, 'sudo', 'sudo', SCRIPT_UNBLOCKINTERNET)
	
## \fn def ejecutar_script(script, parametro=None):
# \brief Función genérica para ejecutar los scripts que sean necesarios.
# \param parametro Puede recibir un parámetro si es requerido para la ejecución del script.
def ejecutar_script(script, parametro=None):
	pid = os.spawnlp(os.P_NOWAIT, 'sudo', 'sudo', script, parametro)
	return pid

## \fn def parar_procesos_activos(pid=None)
# \brief Solicita la finalización los procesos VNC y/o los procesos según su PID
# \param pid PID del proceso que se va a finalizar
def parar_procesos_activos(pid=None):
    finalizar_proceso_x11vnc()
    finalizar_proceso_xvnc4viewer()

    if pid != None:
        finalizar_proceso(pid)
        
## \fn def finalizar_proceso(pid):
# \brief Finaliza un proceso
# \ param PID del proceso que se desea parar.
def finalizar_proceso(pid):
	comando = 'kill -9 '+str(pid)
	os.system(comando)

## \fn def finalizar_proceso_x11vnc():
# \brief Ejecuta la finalización del proceso x11vnc.
def finalizar_proceso_x11vnc():
	os.system(FINALIZAR_PRO_X11VNC)

## \fn def finalizar_proceso_xvnc4viewer():
# \brief Ejecuta la finalización del proceso xvnc4viewer.
def finalizar_proceso_xvnc4viewer():
	os.system(FINALIZAR_PRO_XVNC4VIEWER)

## \fn def lanzar_peticion(comando):
# \brief Ejecuta el comando recibido, en el sistema.
def lanzar_peticion(comando):
	sentencia = 'nohup '+comando+' > /dev/null 2>&1'
	hilo = threading.Thread(target=os.system, args=(sentencia,))
	hilo.daemon = True
	hilo.start()

## \fn 
# \brief Comprueba si el servicio/proceso VNC está activo.
# \ Devuelve True si hay algun servicio activo. False en caso contrario.
# \param ip IP del servidor
def servicio_vnc_activo(ip):
	if not servicio_demo_activo(ip) and not servicio_acceso_desktop(ip):
		return False
	else:
		return True

def servicio_demo_activo(ipCliente):
	try:
		ipServidor = get_ip()
		serverCommand = "netstat -ptna 2>/dev/null | grep x11vnc | grep '"+ipServidor+".*"+ipCliente+"' | grep 'ESTABLECIDO' | awk '{print $7}' | cut -d ""/"" -f 1"
		pid_server = commands.getstatusoutput(serverCommand)

		if not pid_server[1].strip():
			return False
		else:
			return True
			
	except NoDefaultDevice, e:
		lerror("Petición -> Comprobar servicio vnc. No hay conexión y no se pueden realizar algunas funciones necesarias: %s" % e)
		return False

def servicio_acceso_desktop(ipServidor):
	clientCommand = "netstat -ptna 2>/dev/null | grep "+ipServidor+":5905 | grep xvnc4viewer | grep 'ESTABLECIDO' | awk '{print $7}' | cut -d ""/"" -f 1" 
	pid_client = commands.getstatusoutput(clientCommand)

	if not pid_client[1].strip():
		return False
	else:
		return True