#!/usr/bin/python
# -*- encoding: utf-8 -*-
## \namespace gui_controldesktop
from PyQt4 import QtCore, QtGui
import logging
import os
import shutil

from hga.controldesktop.settings import get_ConsultaDemo
from hga.controldesktop.settings import get_ConsultaAcceso
from hga.controldesktop.settings import KEY_DEMOGRUPO
from hga.controldesktop.settings import KEY_DEMOALUM
from hga.controldesktop.settings import KEY_ACCESO
from hga.controlgrupos.common.user_info import get_ip
from hga.controlgrupos.common.exception import NoDefaultDevice
from hga.controlgrupos.common.settings import LOGGING_CONFIG_FILE
from hga.controlgrupos.common.settings import HOMEDIR

from hga.controldesktop.gui_controldesktop.ventana_abrir_navegador import VentanaAbrirNavegador
from hga.controldesktop.gui_controldesktop.ventana_ejecutar_app import VentanaEjecutarAPP
from hga.controldesktop.gui_controldesktop.ventana_msg_info import VentanaMensajeInfo
from hga.controldesktop.gui_controldesktop.ventana_lockgrupo import VentanaLockGrupo

logging.config.fileConfig(LOGGING_CONFIG_FILE)
lerror = logging.error

class sControlDesktop():
	def __init__(self, gui):
		self.gui = gui
		self.lockList = dict()
		self.listaInternet = dict()
		self.msg_demo_return = False # Variable para controlar si el servicio demostración ha sido solicitado.
		self.subclaseLock = None
		self.demoAlumnoActivo = None # Contendrá el alumno al que se le realiza la demostración.
		self.alumnoEmisionGrupo = None # Contendrá el alumno que está realizando la demostración al resto del grupo.
		self.demo_grupo_status = False # Estado de la demostración de grupo, si está o no activa.
		self.funcion_statusBtnBloq = False # Permite conocer desde donde se está emitiendo la señal de btnBloquearAlum
		self.blockInternet_grupo_status = False # Estado del boton bloquear internet de grupo, si está o no activa.
		self.crear_menu_contextual()

	def controldesktop_signals(self):
		# Menú contextual
		self.gui.connect(self.gui.panelControl, QtCore.SIGNAL("customContextMenuRequested(QPoint)"), self.menu_contextual)
		# Apagar equipo remoto
		self.gui.connect(self.gui.btnApagarAlum, QtCore.SIGNAL("clicked()"), self.apagar_equipo)
		self.gui.connect(self.gui.btnApagar, QtCore.SIGNAL("clicked()"), self.apagar_equipo_grupo)
		# Bloquear pantalla remota
		self.gui.connect(self.gui.btnBloquearAlum, QtCore.SIGNAL("toggled(bool)"), self.bloquear_pantalla)
		self.gui.connect(self.gui.btnBloquear, QtCore.SIGNAL("clicked()"), self.bloquear_pantalla_grupo)
		# Reiniciar equipo remoto
		self.gui.connect(self.gui.btnReiniciarAlum, QtCore.SIGNAL("clicked()"), self.reiniciar_equipo)
		self.gui.connect(self.gui.btnReiniciar, QtCore.SIGNAL("clicked()"), self.reiniciar_equipo_grupo)
		# Ejecutar aplicación en equipos remotos
		self.gui.connect(self.gui.btnEjecutarOtraAlum, QtCore.SIGNAL("clicked()"), self.ejecutar_aplicacion)
		self.gui.connect(self.gui.btnEjecutarOtra, QtCore.SIGNAL("clicked()"), self.ejecutar_aplicacion_grupo)
		# Ejecutar navegador en equipos remotos
		self.gui.connect(self.gui.btnEjecutarNavAlum, QtCore.SIGNAL("clicked()"), self.ejecutar_navegador)
		self.gui.connect(self.gui.btnEjecutarNav, QtCore.SIGNAL("clicked()"), self.ejecutar_navegador_grupo)
		# Ejecutar open office en equipos remotos
		self.gui.connect(self.gui.btnEjecutarOfficeAlum, QtCore.SIGNAL("clicked()"), self.abrir_open_office)
		self.gui.connect(self.gui.btnEjecutaroffice, QtCore.SIGNAL("clicked()"), self.abrir_open_office_grupo)
		# Solicitar el control remoto a un equipo
		self.gui.connect(self.gui.btnControl, QtCore.SIGNAL("clicked()"), self.solicitar_control_remoto)
		# Solicitar la visualización de un equipo remoto
		self.gui.connect(self.gui.btnVisualizar, QtCore.SIGNAL("clicked()"), self.solicitar_acceso_remoto)
		# Solicitar la demostracion de un alumno al resto del grupo
		self.gui.connect(self.gui.btnEmiAluGrupo, QtCore.SIGNAL("toggled(bool)"), self.solicitar_emision_alumnoGrupo)
		# Solicita realizar una demostración al grupo o a un usuario concreto
		self.gui.connect(self.gui.btnDemostracion, QtCore.SIGNAL("toggled(bool)"), self.realizar_demostracion_grupo)
		self.gui.connect(self.gui.btnDemostracionAlum, QtCore.SIGNAL("toggled(bool)"), self.realizar_demostracion_usuario)
		# Se abrirá un nuevo diálogo para que el servidor envíe un mensaje informativo al cliente
		self.gui.connect(self.gui.btnMensajeAlum, QtCore.SIGNAL("clicked()"), self.enviar_mensaje_info)
		self.gui.connect(self.gui.btnMensaje, QtCore.SIGNAL("clicked()"), self.enviar_mensaje_info_grupo)
		# Bloquea el acceso a internet a los alumnos o a el alumno seleccionado
		self.gui.connect(self.gui.btnBloqNavAlum, QtCore.SIGNAL("toggled(bool)"), self.bloquear_internet_alumno)
		self.gui.connect(self.gui.btnBloqNav, QtCore.SIGNAL("toggled(bool)"), self.bloquear_internet_grupo)

		self.gui.connect(self.gui, QtCore.SIGNAL("estadoDemo"), self.demo_alumno_activa)
		self.gui.connect(self.gui, QtCore.SIGNAL("estadoDemoGrupo"), self.demo_grupo_activa)
		self.gui.connect(self.gui, QtCore.SIGNAL("estadoEmisionAluGrupo"), self.emision_alugrupo_estado)

		# Solicita captura del escritorio del alumno
		self.gui.connect(self.gui.btnEscritorioAlum, QtCore.SIGNAL("clicked()"), self.escritorio_alumno)
		self.gui.connect(self.gui, QtCore.SIGNAL("dialogoEscritorio"), self.ventana_guardar_escritorio)

	def apagar_equipo(self):
		if self.gui.userSelected_panelControl != None:
			salida = self.gui._message(u"Apagará el equipo de %s, ¿desea continuar?"%self.gui.userSelected_panelControl[0], "question")
			if salida == 1:	
				map(self.gui.controldesktop_handlers["APAGAR_EQUIPO"], self.gui.userSelected_panelControl)
	
	def apagar_equipo_grupo(self):
		salida = self.gui._message(u"Apagará el equipo de todos los miembros del grupo, ¿desea continuar?", "question")
		if salida == 1:	
			self.gui.controldesktop_handlers["APAGAR_EQUIPO"]()

	def reiniciar_equipo(self):
		if self.gui.userSelected_panelControl != None:
			salida = self.gui._message(u"Reiniciará el equipo de %s, ¿desea continuar?"%self.gui.userSelected_panelControl[0], "question")
			if salida == 1:			
				map(self.gui.controldesktop_handlers["REINICIAR_EQUIPO"], self.gui.userSelected_panelControl)
	
	def reiniciar_equipo_grupo(self):
		salida = self.gui._message(u"Reiniciará el equipo de todos los miembros del grupo, ¿desea continuar?", "question")
		if salida == 1:	
			self.gui.controldesktop_handlers["REINICIAR_EQUIPO"]()

	def abrir_open_office(self):
		if self.gui.userSelected_panelControl != None:
			map(self.gui.controldesktop_handlers["ABRIR_OPEN_OFFICE"], self.gui.userSelected_panelControl)
	
	def abrir_open_office_grupo(self):
		self.gui.controldesktop_handlers["ABRIR_OPEN_OFFICE"]()

	def enviar_mensaje_info(self):
		subclase = VentanaMensajeInfo(self.gui, self.gui.userSelected_panelControl)
		subclase.exec_()

	def enviar_mensaje_info_grupo(self):
		subclase = VentanaMensajeInfo(self.gui)
		subclase.exec_()

	def ejecutar_aplicacion(self):
		subclase = VentanaEjecutarAPP(self.gui, self.gui.userSelected_panelControl)
		subclase.exec_()

	def ejecutar_aplicacion_grupo(self):
		subclase = VentanaEjecutarAPP(self.gui)
		subclase.exec_()

	def ejecutar_navegador(self):
		subclase = VentanaAbrirNavegador(self.gui, self.gui.userSelected_panelControl)
		subclase.exec_()

	def ejecutar_navegador_grupo(self):
		subclase = VentanaAbrirNavegador(self.gui)
		subclase.exec_()

	## \fn escritorio_alumno(self)
	# \brief Solicitud para hacer captura de pantalla a un alumno en concreto del grupo
	def escritorio_alumno(self):
		nick = self.gui.userSelected_panelControl[0]
		self.gui.controldesktop_handlers["ESCRITORIO_ALUMNO"](nick)

	## \fn ventana_guardar_escritorio(self, fichero)
	# \brief Metodo encargado de mostrar un dialogo para que el usuario pueda guardar la imagen donde desee
	def ventana_guardar_escritorio(self, fichero):
		ruta = os.environ.get('HOME') + os.path.sep + "Descargas"
		explorer = QtGui.QFileDialog.getSaveFileName(self.gui, 'Guardar archivo', ruta, "All files(*)")
		if explorer != '':
			shutil.copy(str(fichero), explorer)
			self.gui._message(u"<br>El archivo se ha guardado correctamente.","information")

	## \fn def bloquear_internet_alumno(self, status)
	# \brief Función que recibe el estado del servicio bloquear internet (individual)
	# \param status Estado del servicio
	def bloquear_internet_alumno(self, status):
		if bool(self.gui.userSelected_panelControl):
			nick = self.gui.userSelected_panelControl[0]
			if status == True:
				self.listaInternet[nick] = True
				map(self.gui.controldesktop_handlers["BLOQUEAR_INTERNET"], self.gui.userSelected_panelControl)
			else:
				self.listaInternet[nick] = False
				map(self.gui.controldesktop_handlers["DESBLOQUEAR_INTERNET"], self.gui.userSelected_panelControl)
			self.estado_botones_bloqInternet()

	## \fn def envia_senal_bloqInt_user(self, user)
	# \brief Función que controla si un usuario nuevo debe desactivar internet, y cuando se termine o se salga 
	# de la apli, se dejen los equipos como estaban
	# \param user usuario al que se le envia el mensaje
	# \param accion accion bloquear/desbloquear
	def envia_senal_bloqInt_user(self, user, accion):
		if accion == True:
			self.listaInternet[user] = accion
			self.gui.controldesktop_handlers["BLOQUEAR_INTERNET"](user)
		else:
			self.gui.controldesktop_handlers["DESBLOQUEAR_INTERNET"](user)

	## \fn def bloquear_internet_grupo(self, status)
	# \brief Función que recibe el estado del servicio bloquear internet (grupo)
	# \param status Estado del servicio
	def bloquear_internet_grupo(self, status):	
		if  status == True:
			self.gui.controldesktop_handlers["BLOQUEAR_INTERNET"]()
			self.generarListaUser(True)
		else:
			self.gui.controldesktop_handlers["DESBLOQUEAR_INTERNET"]()
			self.generarListaUser(False)

		self.bloqInt_grupo_activa(status)

	## \fn def generarListaUser(self, accion)
	# \brief Función que recibe el estado para los usuarios, carga la lista de los mismos y les atribuye el valor
	# \param accion Bloqueado o desbloqueado
	def generarListaUser(self, accion):
		usuarios = dict( [(u, self.gui.good_users_conex[u]) for u in self.gui.good_users_conex if not self.gui.good_users_conex[u]["banned"] ])
		for nick in usuarios.keys():
			self.listaInternet[nick] = accion

	## \fn def bloqInt_grupo_activa(self, estado)
	# \brief Función que recibe el estado del servicio bloquear_internet (grupo), para administrar los elementos de la gui según corresponda
	# \param estado Estado del servicio
	def bloqInt_grupo_activa(self, estado):
		self.blockInternet_grupo_status = estado
		if not estado:
			self.gui.btnBloqNav.setIcon(self.gui.iconBlockInt)
			self.gui.btnBloqNav.setToolTip(u"Bloquear Internet")
			self.gui.btnBloqNav.setChecked(False)
		else:
			self.gui.btnBloqNav.setIcon(self.gui.iconDesblockInt)
			self.gui.btnBloqNav.setToolTip(u"Desbloquear Internet")
			self.gui.btnBloqNav.setChecked(True)

	## \fn def estado_botones_bloqInternet(self)
	# \brief Función que controla el estado de los botones en relacion al bloqueo de internet
	# \param estado Estado del servicio
	def estado_botones_bloqInternet(self):
		if bool(self.gui.userSelected_panelControl):
			nick = self.gui.userSelected_panelControl[0]
			if nick in self.listaInternet and self.listaInternet[nick] == True:
				self.gui.btnBloqNavAlum.setIcon(self.gui.iconDesblockIntG)
				self.gui.btnBloqNavAlum.setChecked(True)
				self.gui.btnBloqNavAlum.setToolTip("Desbloquear internet")
				self.menu_bloqueo_internet.setText("Desbloquear internet")
				self.menu_bloqueo_internet.setChecked(True)
			elif nick not in self.listaInternet or self.listaInternet[nick] == False:
				self.gui.btnBloqNavAlum.setIcon(self.gui.iconBlockIntG)
				self.gui.btnBloqNavAlum.setChecked(False)
				self.gui.btnBloqNavAlum.setToolTip("Bloquear internet")
				self.menu_bloqueo_internet.setText("Bloquear internet")
				self.menu_bloqueo_internet.setChecked(False)

	## \fn def actualizar_locklistInternet(self, users):
	# \brief Actualiza la lista de usuarios. Si algún usuario ya no existe, lo borra.
	#  \param users Lista de usuarios conectados
	def actualizar_locklistInternet(self, users):
		for nick in self.listaInternet.keys():
			if not nick in users and nick in self.listaInternet:
				del self.listaInternet[nick]

	def solicitar_control_remoto(self):
		if self.gui.procesos_progressBar == None:
			if not self.demo_grupo_status and self.demoAlumnoActivo != self.gui.userSelected_panelControl and self.alumnoEmisionGrupo == None:
				self.emitir_startProgressBar(KEY_ACCESO, self.gui.userSelected_panelControl[0])
				map(self.gui.controldesktop_handlers["SOL_CONTROL_REMOTO"],self.gui.userSelected_panelControl)
			else:
				if self.demo_grupo_status:
					self.gui._message(u"No puede activar el acceso remoto mientras realiza una demostración de grupo.", "information")
				if self.demoAlumnoActivo == self.gui.userSelected_panelControl:
					self.gui._message(u"No puede activar el acceso remoto a %s, mientras realiza una demostración." % self.gui.userSelected_panelControl[0], "information")
				if self.alumnoEmisionGrupo != None:
					self.gui._message(u"El usuario %s está emitiendo al grupo." %self.alumnoEmisionGrupo[0], "information")

	def solicitar_acceso_remoto(self):
		if self.gui.procesos_progressBar == None:
			if not self.demo_grupo_status and self.demoAlumnoActivo != self.gui.userSelected_panelControl and self.alumnoEmisionGrupo == None:
				self.emitir_startProgressBar(KEY_ACCESO, self.gui.userSelected_panelControl[0])
				map(self.gui.controldesktop_handlers["SOL_ACCESO_REMOTO"],self.gui.userSelected_panelControl)
			else:
				if self.demo_grupo_status:
					self.gui._message(u"No puede activar el acceso remoto mientras realiza una demostración de grupo.", "information")
				if self.demoAlumnoActivo == self.gui.userSelected_panelControl:
					self.gui._message(u"No puede activar el acceso remoto a %s, mientras realiza una demostración." % self.gui.userSelected_panelControl[0], "information")
				if self.alumnoEmisionGrupo != None:
					self.gui._message(u"El usuario %s está emitiendo al grupo." %self.alumnoEmisionGrupo[0], "information")

	def realizar_demostracion_usuario(self, status):
		if self.gui.procesos_progressBar == None:
			if self.msg_demo_return == False:
				if not self.demo_grupo_status:
					if self.gui.userSelected_panelControl != None:
						if status == True:
							self.demoAlumnoActivo = self.gui.userSelected_panelControl
							self.emitir_startProgressBar(KEY_DEMOALUM,self.gui.userSelected_panelControl[0])
							map(self.gui.controldesktop_handlers["DEMO_USER"],self.gui.userSelected_panelControl)
						else:
							if self.gui.procesos_progressBar != None and self.gui.procesoKey == KEY_DEMOALUM:
								self.gui.procesos_progressBar.terminate
							self.gui.controldesktop_handlers["STOP_DEMO"]()
							self.demoAlumnoActivo = None
				else:
					self.demoAlumnoActivo = None
		self.estado_botones_demo()
		self.gui.estado_botones_userList()

	def realizar_demostracion_grupo(self, status):
		if self.gui.procesos_progressBar == None:
			if self.demo_grupo_status != status and self.demo_alumno_activa != None:	
				if  status == True:
					self.emitir_startProgressBar(KEY_DEMOGRUPO)
					self.gui.controldesktop_handlers["DEMO_GRUPO"]()
				else:
					if self.gui.procesos_progressBar != None and self.gui.procesoKey == KEY_DEMOGRUPO:
						self.gui.procesos_progressBar.terminate
					self.gui.controldesktop_handlers["STOP_DEMO"]()
				self.gui.estado_botones_userList()

	## \fn def demo_alumno_activa(self, estado):
	# \brief Función que recibe el estado del servicio demostración (individual), para administrar los elementos de la gui según corresponda
	# \param estado Estado del servicio
	def demo_alumno_activa(self, estado):
		if not estado:
			self.demoAlumnoActivo = None
			self.msg_demo_return = True
		self.estado_botones_demo()

	## \fn def demo_grupo_activa(self,estado)
	# \brief Función que recibe el estado del servicio demostración (grupo), para administrar los elementos de la gui según corresponda
	# \param estado Estado del servicio
	def demo_grupo_activa(self,estado):
		self.demo_grupo_status = estado
		if not estado:
			self.gui.btnDemostracion.setIcon(self.gui.icon31)
			self.gui.btnDemostracion.setToolTip(u"Hacer demostración")
			self.gui.btnDemostracion.setChecked(False)
		else:
			self.gui.btnDemostracion.setIcon(self.gui.icon32)
			self.gui.btnDemostracion.setToolTip(u"Parar demostración")
			self.gui.btnDemostracion.setChecked(True)

	def solicitar_emision_alumnoGrupo(self, status):
		if self.gui.procesos_progressBar == None:
			if self.msg_demo_return == False:
				if self.demo_grupo_status == False and self.demoAlumnoActivo == None:
					if self.gui.userSelected_panelControl != None:
						if status == True:
							self.alumnoEmisionGrupo = self.gui.userSelected_panelControl
							map(self.gui.controldesktop_handlers["SOL_EMISION_ALUGRUPO"],self.gui.userSelected_panelControl)
						else:
							map(self.gui.controldesktop_handlers["STOP_EMISION_ALUGRUPO"],self.gui.userSelected_panelControl)
							self.alumnoEmisionGrupo = None

				self.estado_botones_demo()

	## \fn def emision_alugrupo_estado(self,estado)
	# \brief Función que recibe el estado del servicio emisión de un usuario al grupo, para administrar los elementos de la gui según corresponda
	# \param estado Estado del servicio
	def emision_alugrupo_estado(self,estado):
		self.msg_demo_return = True
		if not estado:
			self.alumnoEmisionGrupo = None
		self.estado_botones_demo()

	## \fn def estado_botones_demo(self, habilitarDemoGrupo):
	# \brief Gestiona el estado de los botones de la funcionalidad demostración
	# \brief Estado que se solicita para el botón demostración de grupo.
	def estado_botones_demo(self):
		if self.demo_grupo_status == False:
			# Si no existe una demo activa, se mantienen los botones de accion individual en el estado inicial
			if self.demoAlumnoActivo == None:
				self.gui.btnDemostracionAlum.setIcon(self.gui.icon33)
				self.gui.btnDemostracionAlum.setChecked(False)
				self.gui.btnDemostracionAlum.setToolTip(u"Hacer demostración")
				self.menu_demostracion.setText(u"Hacer demostración")
				self.menu_demostracion.setChecked(False)
				if self.alumnoEmisionGrupo == None:
					self.gui.btnDemostracionAlum.setEnabled(True)
					self.menu_demostracion.setEnabled(True)
				
				else:
					self.gui.btnDemostracionAlum.setEnabled(False)
					self.menu_demostracion.setEnabled(False)

			# Si la demo activa se está realizando sobre el usuario seleccionado
			elif self.demoAlumnoActivo != None and self.demoAlumnoActivo == self.gui.userSelected_panelControl:
				self.gui.btnDemostracionAlum.setIcon(self.gui.icon34)
				self.gui.btnDemostracionAlum.setEnabled(True)
				self.gui.btnDemostracionAlum.setChecked(True)
				self.gui.btnDemostracionAlum.setToolTip(u"Parar demostración")
				self.menu_demostracion.setText(u"Parar demostración")
				self.menu_demostracion.setChecked(True)
				self.menu_demostracion.setEnabled(True)

			# Si el usuario seleccionado no es el usuario con la demostración activa
			elif self.demoAlumnoActivo != None and self.demoAlumnoActivo != self.gui.userSelected_panelControl:
				self.gui.btnDemostracionAlum.setIcon(self.gui.icon33)
				self.gui.btnDemostracionAlum.setEnabled(False)
				self.gui.btnDemostracionAlum.setToolTip(u"Hacer demostración")
				self.menu_demostracion.setText(u"Hacer demostración")
				self.menu_demostracion.setChecked(False)
				self.menu_demostracion.setEnabled(False)


			if self.alumnoEmisionGrupo == None:
				self.gui.btnEmiAluGrupo.setIcon(self.gui.iconDemoAluGrupo)
				self.gui.btnEmiAluGrupo.setChecked(False)
				self.gui.btnEmiAluGrupo.setToolTip(u"Emitir pantalla de alumno")
				self.menu_emisionAlumno.setText(u"Emitir pantalla de alumno")
				self.menu_emisionAlumno.setChecked(False)
				
				# Se habilitarian los botones de visualizar y control si se pide, cuando está desactivada la función emitir.
				#self.gui.btnControl.setEnabled(True);
				#self.gui.btnVisualizar.setEnabled(True);
				#self.control_alum.setEnabled(True);
				#self.visualizar.setEnabled(True);


				if self.demoAlumnoActivo == None:
					self.gui.btnEmiAluGrupo.setEnabled(True)
					self.menu_emisionAlumno.setEnabled(True)
				else:
					self.gui.btnEmiAluGrupo.setEnabled(False)
					self.menu_emisionAlumno.setEnabled(False)

			elif self.alumnoEmisionGrupo != None and self.alumnoEmisionGrupo == self.gui.userSelected_panelControl:
				self.gui.btnEmiAluGrupo.setIcon(self.gui.icon34)
				self.gui.btnEmiAluGrupo.setEnabled(True)
				self.gui.btnEmiAluGrupo.setChecked(True)
				self.gui.btnEmiAluGrupo.setToolTip(u"Parar emisión pantalla de alumno")
				self.menu_emisionAlumno.setText(u"Parar emisión pantalla de alumno")
				self.menu_emisionAlumno.setEnabled(True)
				self.menu_emisionAlumno.setChecked(True)

				# Se deshabilitarian los botones visualizar y controlar si se pide, cuando está activada la función emitir.
				#self.gui.btnControl.setEnabled(False);
				#self.gui.btnVisualizar.setEnabled(False);
				#self.control_alum.setEnabled(False);
				#self.visualizar.setEnabled(False);

			elif self.alumnoEmisionGrupo != None and self.alumnoEmisionGrupo != self.gui.userSelected_panelControl:
				self.gui.btnEmiAluGrupo.setIcon(self.gui.iconDemoAluGrupo)
				self.gui.btnEmiAluGrupo.setEnabled(False)
				self.gui.btnEmiAluGrupo.setToolTip(u"Emitir pantalla de alumno")
				self.menu_emisionAlumno.setText(u"Emitir pantalla de alumno")
				self.menu_emisionAlumno.setEnabled(False)
				self.menu_emisionAlumno.setChecked(False)

				# Se deshabilitarian los botones visualizar y controlar si se pide, cuando está activada la función emitir.
				#self.gui.btnControl.setEnabled(False);
				#self.gui.btnVisualizar.setEnabled(False);
				#self.control_alum.setEnabled(False);
				#self.visualizar.setEnabled(False);

		else: # Si se está realizando una demostración al grupo los botones de acción individual se mantendrán deshabilitados.
			self.menu_demostracion.setEnabled(False)
			self.gui.btnDemostracionAlum.setEnabled(False)
			self.gui.btnEmiAluGrupo.setEnabled(False)
			self.menu_emisionAlumno.setEnabled(False)

		if self.gui.fichaUsuario == True or self.gui.reset_gui == True:
			self.gui.btnDemostracion.setEnabled(False)
		else:
			if self.demoAlumnoActivo == None and self.alumnoEmisionGrupo == None:
				self.gui.btnDemostracion.setEnabled(True)
			else:
				self.gui.btnDemostracion.setEnabled(False)

		self.msg_demo_return = False

	def bloquear_pantalla_grupo(self):
		self.subclaseLock = VentanaLockGrupo(self, self.gui.good_users_conex)
		self.subclaseLock.exec_()

	def bloquear_pantalla(self, status):
		if self.funcion_statusBtnBloq == False:
			if bool(self.gui.userSelected_panelControl):
				nick = self.gui.userSelected_panelControl[0]
				if status == True:
					self.lockList[nick] = True
					map(self.gui.controldesktop_handlers["BLOQUEAR_PANTALLA"],self.gui.userSelected_panelControl)
				else:
					self.lockList[nick] = False
					map(self.gui.controldesktop_handlers["DESBLOQUEAR_PANTALLA"],self.gui.userSelected_panelControl)
				self.estado_botones_lockscreen()
			else:
				self.gui._message(u"Seleccione un usuario.","information")

	## \fn def estado_botones_lockscreen(self):
	# \brief Gestiona el estado de los botones de la funcionalidad bloquear para mostrarlos en función del estado de bloqueo del usuario.
	def estado_botones_lockscreen(self):
		self.funcion_statusBtnBloq = True
		if bool(self.lockList):
			if bool(self.gui.userSelected_panelControl):
				nick = self.gui.userSelected_panelControl[0]
				if nick in self.lockList and self.lockList[nick] == True:
					self.gui.btnBloquearAlum.setIcon(self.gui.icon43)
					self.gui.btnBloquearAlum.setChecked(True)
					self.gui.btnBloquearAlum.setToolTip("Desbloquear equipo")
					self.menu_bloqueo.setText("Desbloquear equipo")
					self.menu_bloqueo.setChecked(True)
				elif nick not in self.lockList or self.lockList[nick] == False:
					self.gui.btnBloquearAlum.setIcon(self.gui.icon42)
					self.gui.btnBloquearAlum.setChecked(False)
					self.gui.btnBloquearAlum.setToolTip("Bloquear equipo")
					self.menu_bloqueo.setText("Bloquear equipo")
					self.menu_bloqueo.setChecked(False)
		else:
			self.gui.btnBloquearAlum.setIcon(self.gui.icon42)
			self.gui.btnBloquearAlum.setChecked(False)
			self.gui.btnBloquearAlum.setToolTip("Bloquear equipo")
			self.menu_bloqueo.setText("Bloquear equipo")
			self.menu_bloqueo.setChecked(False)
		self.funcion_statusBtnBloq = False


	## \fn def usuario_bloqueado(self, user):
	# \brief Función que comprueba si el usuario que le llega está bloqueado o no
	#  \param user usuario
	def usuario_bloqueado(self, user):
		if user in self.lockList and self.lockList[user] == True:
			return True
		else:
			return False

	## \fn def actualizar_locklist(self, users):
	# \brief Actualiza la lista de usuarios bloqueados. Si algún usuario ya no existe, lo borra.
	#  \param users Lista de usuarios conectados
	def actualizar_locklist(self, users):
		for nick in self.lockList.keys():
			if not nick in users and nick in self.lockList: #No tenemos en cuenta los usuarios baneados.
				del self.lockList[nick]
		# Comprueba si existe la ventana de usuarios bloqueados (bloquear grupo) y la actualiza.
		if self.subclaseLock != None:
			self.subclaseLock.emit(QtCore.SIGNAL('generarLista'), users)

	def menu_contextual(self, QPos):
		user = [unicode(i.text()) for i in self.gui.panelControl.selectedItems()]
		self.gui.userSelected_panelControl = user
		if bool(self.gui.userSelected_panelControl):
			self.estado_botones_lockscreen()
			self.estado_botones_demo()
			self.estado_botones_bloqInternet()
			parenPosition = QtCore.QPoint()
			parenPosition = self.gui.panelControl.mapToGlobal(QtCore.QPoint(0,5))
			self.menu.move(parenPosition + QPos)
			self.menu.show()

	def crear_menu_contextual(self):
		self.menu = QtGui.QMenu(self.gui)

		self.control_alum = QtGui.QAction('Tomar el control', self.gui)
		self.control_alum.triggered.connect(self.solicitar_control_remoto)
		self.menu.addAction(self.control_alum)

		self.visualizar = QtGui.QAction('Visualizar pantalla', self.gui)
		self.visualizar.triggered.connect(self.solicitar_acceso_remoto)
		self.menu.addAction(self.visualizar)

		self.menu_demostracion = QtGui.QAction(u'Hacer demostración', self.gui)
		self.menu_demostracion.setCheckable(True)
		self.gui.connect(self.menu_demostracion, QtCore.SIGNAL("triggered()"), lambda: self.realizar_demostracion_usuario(self.menu_demostracion.isChecked()))
		self.menu.addAction(self.menu_demostracion)

		self.menu_emisionAlumno = QtGui.QAction(u'Emitir pantalla de alumno', self.gui)
		self.menu_emisionAlumno.setCheckable(True)
		self.gui.connect(self.menu_emisionAlumno, QtCore.SIGNAL("triggered()"), lambda: self.solicitar_emision_alumnoGrupo(self.menu_emisionAlumno.isChecked()))
		self.menu.addAction(self.menu_emisionAlumno)

		self.menu.addSeparator()

		ejec_nav = QtGui.QAction('Ejecutar navegador', self.gui)
		ejec_nav.triggered.connect(self.ejecutar_navegador)
		self.menu.addAction(ejec_nav)

		self.menu_bloqueo_internet = QtGui.QAction('Bloquear internet', self.gui)
		self.menu_bloqueo_internet.setCheckable(True)
		self.gui.connect(self.menu_bloqueo_internet, QtCore.SIGNAL("triggered()"), lambda: self.bloquear_internet_alumno(self.menu_bloqueo_internet.isChecked()))
		self.menu.addAction(self.menu_bloqueo_internet)

		open_office = QtGui.QAction('Abrir LibreOffice Writer', self.gui)
		open_office.triggered.connect(self.abrir_open_office)
		self.menu.addAction(open_office)

		ejec_apli = QtGui.QAction(u'Ejecutar aplicación', self.gui)
		ejec_apli.triggered.connect(self.ejecutar_aplicacion)
		self.menu.addAction(ejec_apli)

		# Esta funcionalidad no pertenece a este módulo, pero se incluye aquí por comodidad
		# para la gestión del menú
		multimedia = QtGui.QAction('Enviar archivos multimedia', self.gui)
		multimedia.triggered.connect(self.gui.emitir_streaming_individual)
		self.menu.addAction(multimedia)

		sms = QtGui.QAction('Enviar mensaje', self.gui)
		sms.triggered.connect(self.enviar_mensaje_info)
		self.menu.addAction(sms)

		escritorioAlum = QtGui.QAction('Realizar captura', self.gui)
		escritorioAlum.triggered.connect(self.escritorio_alumno)
		self.menu.addAction(escritorioAlum)

		self.menu.addSeparator()

		apagar = QtGui.QAction('Apagar equipo', self.gui)
		apagar.triggered.connect(self.apagar_equipo)
		self.menu.addAction(apagar)

		reiniciar = QtGui.QAction('Reiniciar equipo', self.gui)
		reiniciar.triggered.connect(self.reiniciar_equipo)
		self.menu.addAction(reiniciar)

		self.menu_bloqueo = QtGui.QAction('Bloquear equipo', self.gui)
		self.menu_bloqueo.setCheckable(True)
		self.gui.connect(self.menu_bloqueo, QtCore.SIGNAL("triggered()"), lambda: self.bloquear_pantalla(self.menu_bloqueo.isChecked()))
		self.menu.addAction(self.menu_bloqueo)

	## \fn def emitir_startProgressBar(self, procesoKey, nick=None)
	# \brief Gestiona la petición de ejecutar la barra de progreso según la funcionalidad que la solicite.
	# \param procesoKey Clave de la funcionalidad que realiza la petición
	# \param nick Nick del usuario si la funcionalidad lo requiere
	def emitir_startProgressBar(self, procesoKey, nick=None):
		if nick != None:
			ip = self.gui.users[nick].get("IP")
		if procesoKey == KEY_ACCESO: # Control o visualización de escritorio remoto
			consulta = get_ConsultaAcceso(ip)
			self.gui.emit(QtCore.SIGNAL("ProgressBar"), procesoKey, consulta, nick)
		elif procesoKey == KEY_DEMOALUM: # Demostración del servidor a un cliente.
			try:
				ip = get_ip()
				consulta = get_ConsultaDemo(ip)
				self.gui.emit(QtCore.SIGNAL("ProgressBar"), procesoKey, consulta, nick)
			except NoDefaultDevice, e:
				lerror("sControlDesktop -> emitir_startProgressBar. No hay conexión: %s" % e)
		elif procesoKey == KEY_DEMOGRUPO: # Demostración del servidor al grupo
			try:
				ip = get_ip()
				consulta = get_ConsultaDemo(ip)
				self.gui.emit(QtCore.SIGNAL("ProgressBar"), procesoKey, consulta)
			except NoDefaultDevice, e:
				lerror("sControlDesktop -> emitir_startProgressBar. No hay conexión: %s" % e)