#!/usr/bin/python
# -*- encoding: utf-8 -*-
## \namespace gui_controleditor
from PyQt4 import QtCore, QtGui, QtWebKit
import os, time
from hga.controleditor.gui_controleditor.ventana_abrir_pad import VentanaAbrirPad
from hga.controlgrupos.common.settings import HOMEDIR


class sControlEditor_server():
	def __init__(self, gui):
		self.gui = gui
		self.padActual = ""

		self.gui.webview.page().setLinkDelegationPolicy(QtWebKit.QWebPage.DelegateAllLinks)

	def controleditor_signals(self):
		# Señales correspondientes a la herramienta de desarrollo colaborativo
		self.gui.connect(self.gui.HColaborativa, QtCore.SIGNAL("triggered()"), lambda: self.activarHerramienta(self.gui.HColaborativa.isChecked()))
		self.gui.connect(self.gui.actionAbrirPads, QtCore.SIGNAL("triggered()"), self.ventana_nomPad)
		self.gui.connect(self.gui, QtCore.SIGNAL("serverStart"), self.serverStart)
		self.gui.connect(self.gui, QtCore.SIGNAL("deshabilitar_HC"), self.stopWBot)

		#parte nueva del webview *****
		self.gui.connect(self.gui.actionVerPads, QtCore.SIGNAL("triggered()"), self.listar_pads)
		self.gui.connect(self.gui.actionExportarPads, QtCore.SIGNAL("triggered()"), self.exportar_pads)
		self.gui.connect(self.gui.actionImprimirPads, QtCore.SIGNAL("triggered()"), self.imprimir_pads)
		self.gui.connect(self.gui.webview, QtCore.SIGNAL("urlChanged (const QUrl&)"), self.url_changed)
		self.gui.connect(self.gui.webview, QtCore.SIGNAL("linkClicked (const QUrl&)"), self.link_clicked)

	""" """"""""""""""""""""""""""""""""""""""""""" """
	"""   "Funciones de Herramienta Colaborativa"   """
	""" """"""""""""""""""""""""""""""""""""""""""" """
	## \fn def activarHerramienta(self, status):
	# \brief Función que solicita al core el arranque o la parada del servidor.
	# \param status Éstado del QAction al ser pulsado. 
	def activarHerramienta(self, status):
		if self.gui.procesos_progressBar == None:
			if status:
				self.gui.controleditor_handlers["ACT_SERVER_CONEDIT"]("START")
				self.gui.HColaborativa.setEnabled(False)
			else:
				self.gui.controleditor_handlers["ACT_SERVER_CONEDIT"]("STOP")

	## \fn def stopWBot(self):
	# \brief Función que solicita al core la parada del servidor cuando no existe conexion.
	def stopWBot(self):
		self.gui.controleditor_handlers["ACT_SERVER_CONEDIT"]("STOP")

	## \fn def serverStart(self, serverStatus):
	# \brief Recibe la señal emitida por el core indicando si el servidor esta activo o parado.
	# \param serverStatus, True = servidor arrancado o False = servidor parado
	def serverStart(self, serverStatus, pid=None):
		if serverStatus:
			self.ventana_nomPad()
			self.gui.HColaborativa.setEnabled(True)
			self.gui.actionVerPads.setEnabled(True)
			self.gui.actionAbrirPads.setEnabled(True)
			self.gui.actionExportarPads.setEnabled(True)
			self.gui.actionImprimirPads.setEnabled(True)
			if pid != None:
				self.gui.controleditor_handlers["PID_EDITOR"](pid)
		else:
			self.gui.actionVerPads.setEnabled(False)
			self.gui.actionAbrirPads.setEnabled(False)
			self.gui.actionExportarPads.setEnabled(False)
			self.gui.actionImprimirPads.setEnabled(False) 
			self.gui.webview.stop()
			self.gui.panelUsuario.setTabEnabled(2,False)

	def deshabilitar_herramienta(self):
		self.gui.HColaborativa.setEnabled(False)
		self.gui.HColaborativa.setChecked(False)
		self.gui.actionVerPads.setEnabled(False)
		self.gui.actionAbrirPads.setEnabled(False)
		self.gui.actionImprimirPads.setEnabled(False)
		self.gui.actionExportarPads.setEnabled(False)
		self.gui.panelUsuario.setCurrentIndex(0)
		self.gui.panelUsuario.setTabEnabled(2,False)

	## \fn def exportar_pads(self):
	# \brief Permite exportar el documento con extension .txt desde el navegador firefox
	def exportar_pads(self):
		os.spawnlp(os.P_NOWAIT,'firefox', 'firefox', str(self.padActual)+'/export/txt')

	## \fn def imprimir_pads(self):
	# \brief Permite imprimir en formato pdf el documento
	def imprimir_pads(self):
		printer = QtGui.QPrinter(QtGui.QPrinter.HighResolution)
		printer.setOutputFormat(QtGui.QPrinter.PdfFormat)
		printer.setOrientation(QtGui.QPrinter.Portrait)
		printer.setPaperSize(QtGui.QPrinter.A4)
		printer.setFullPage(False)
		
		fname = QtGui.QFileDialog(self.gui)
		fname.setFileMode(1)
		target = os.environ.get('HOME') + os.path.sep + "Descargas" 
		fname.setDirectory(QtCore.QString(target))
		fname.setFileMode(QtGui.QFileDialog.AnyFile)
		fname.setLabelText(QtGui.QFileDialog.LookIn,"Ruta del sistema")
		fname.setLabelText(QtGui.QFileDialog.Reject,"Cancelar")
		fname.setLabelText(QtGui.QFileDialog.FileType,"Tipo de extension permitida")
		fname.setLabelText(QtGui.QFileDialog.FileName,"Nombre pdf")
		fname.setLabelText(QtGui.QFileDialog.Accept,"Guardar pdf")
		fname.setFilter("Pdf *.pdf")
		fname.setWindowTitle("Guardar archivo pdf")
		result=fname.exec_()

		if result == 1:
			nombre_fichero = fname.selectedFiles()[0]
			printer.setOutputFileName(nombre_fichero)
			self.gui.webview.setTextSizeMultiplier(2)
			self.gui.webview.print_(printer)
			self.gui.webview.setTextSizeMultiplier(1)

			if printer.PrinterState() == 0:
				time.sleep(3)
				os.spawnlp(os.P_NOWAIT,'evince', 'evince', str(nombre_fichero))

	## \fn def ventana_nomPad(self):
	# \brief Abre un dialogo donde se podrá introducir el nombre del documento al que se quiere acceder 
	def ventana_nomPad(self):
		subclase = VentanaAbrirPad(self)
		subclase.exec_()

	## \fn def acceder_pad(self):
	# \brief Tras haber sido activado el servidor, carga en el webview la URL, permitiendo visualizar el Pad.
	def acceder_pad(self, nomPad):
		self.padActual = 'http://127.0.0.1:9001/p/'+nomPad
		self.gui.webview.setVisible(True)
		self.gui.webview.load(QtCore.QUrl(self.padActual))
		self.gui.panelUsuario.setTabEnabled(2,True)
		self.gui.panelUsuario.setCurrentIndex(2)
		self.gui.controleditor_handlers["MSG_ACCEDER_PAD"](nomPad)

	## \fn def listar_pads(self):
	# \brief Lista todos los documentos guardados en el servidor
	def listar_pads(self):
		pad = 'http://127.0.0.1:9001/pad-lister'
		if not self.gui.panelUsuario.isTabEnabled(2):
			self.gui.panelUsuario.setTabEnabled(2,True)
		self.gui.panelUsuario.setCurrentIndex(2)
		self.gui.webview.setVisible(True)
		self.gui.webview.setUrl(QtCore.QUrl(pad))

	## \fn def url_changed(self):
	# \brief Metodo que emite la señal para los clientes puedan acceder al documento seleccionado por el servidor
	def url_changed(self):
		weblistpad = self.gui.webview.url().toString()
		if not str(weblistpad).endswith('pad-lister'):
			nombreD = str(weblistpad).split('/')
			self.gui.controleditor_handlers["MSG_ACCEDER_PAD"](nombreD[4])

	## \fn def link_clicked(self, url):
	# \brief Metodo muestra el documento seleccionado
	def link_clicked(self, url):
		self.gui.webview.setUrl(QtCore.QUrl(url.toString()))
		self.padActual = url.toString()