#!/usr/bin/env python
# -*- coding: utf-8 -*-
##\namespace alumno
import sys
import os
import logging
import logging.config
import base64
import commands
import time

from threading import Lock
from socket import socket

from PyQt4 import QtGui, QtCore

from dbus.mainloop.glib import DBusGMainLoop

from hga.controlgrupos.common.settings import LOGGING_CONFIG_FILE
from hga.controlgrupos.cliente.settingscliente import EXIT_CODE_MESSAGES
from hga.controldesktop.controlpeticiones import parar_procesos_activos
from hga.controlgrupos.common.user_info import ping
from hga.controlgrupos.common.user_info import get_user_name
from hga.controlgrupos.common.user_info import generate_jid
from hga.controlgrupos.common.user_info import identify_user
from hga.controlgrupos.common.user_info import get_ip
from hga.controlgrupos.common.smdns import SMdns
from hga.controlgrupos.common.exception import NoDefaultDevice

from hga.controlgrupos.cliente.clientealumno import ClienteAlumno

from hga.controlgrupos.common.settings import DESKTOP_RUTA
from hga.controlstreaming.servidor.controlstreaming import parar_vlc

from hga.gui.common.sidentify import identifyWindow

logging.config.fileConfig(LOGGING_CONFIG_FILE)
lerror = logging.error


##\clase AppCoreAlumno
# \informe HGR core Alumno.
class AppCoreAlumno:
	## \fn __init__(self)
	# \informe constructor Core. Monta los handlers, gui, debus, etc..
	def __init__(self, qtapp, gui, stream_handlers):
		self.resource_list = {}
		self.resource_name = None
		self.shared_info = None
		self.bot = None
		self.mdns = None
		self.log = None
		self.nick = None
		self.qtapp = qtapp
		self.appcore_handlers = None
		self.ip_muc = None
		self.ipServidor = None

		self.gui = gui

		self._setup_logging()
		self.log.info(QtGui.QApplication.translate("AppCoreAlumno.log", "Iniciando cliente."))

		self.bot_handlers = {
			"STATUS_CONNECTED": self.handle_connected,
			"STATUS_DISCONNECTED": self.handle_disconnected,
			"ACTUALIZAR_NOM_USER_SERVER": self.actualizar_nom_user_server,
			"ACTUALIZAR_NOM_USER_CLIENT": self.actualizar_nom_user_client,
			"STATUS_MESSAGE":   self.mensaje_estado_muc,
			"SEND_GUI_MSG": self.enviar_gui_msg,
			"CAPTURA": self.realiza_captura,
			"ENVIAR_MENSAJE": self.enviar_mensaje,
			"STOP_HCOLABORATIVA": self.stop_hcolaborativa,
			"RELOAD_URL_HC": self.reload_url_hc,
			"EMISION_VLC": self.emision_vlc,
			"STOP_VLC": self.stop_vlc,
			"HABILITA_AYUDA": self.habilita_ayuda,
			"REALIZAR_IMAGEN_ESCRITORIO": self.realizar_imagen_escritorio,
		}

		#Handlers-cliente para el streaming de video
		self.stream_handlers = stream_handlers

		self.jid_sala = ""
		self.flag_borra_jid = False
		self.vlc_activo = False

		try:
			self._setup_dbus()
		except NoDefaultDevice, e:
			pass
		
		self.resource_list_lock = Lock()

	
	## \fn _setup_logging(self)
	# \brief Establece el atributo de clase de registro.
	def _setup_logging(self):
		logging.config.fileConfig(LOGGING_CONFIG_FILE)
		self.log = logging.getLogger('hgr.client')

	def set_handlers_appcore(self, appcore_handlers):
		self.appcore_handlers = appcore_handlers
	

	## \fn actualizar_nombre_cliente(self)
	# \brief Cambia el nombre del cliente una vez que lo solicita.
	def actualizar_nom_user_client(self):
		if self.vlc_activo == False:
			ventana_identify = identifyWindow()
			ventana_identify.setAttribute(QtCore.Qt.WA_DeleteOnClose)
			salida = ventana_identify.exec_()
			if self.jid_sala != "" and salida == 1:
				self.flag_borra_jid = True
				self.actualizar_nom_user_server()
			self.flag_borra_jid = False
		else:
			self.enviar_gui_msg('No puede cambiar de nombre mientras visualiza un video.','information')

	def reconectar(self):
		if not self.resource_list:
			try:
				self._setup_dbus()
			except NoDefaultDevice, e:
				print e
				
	## \fn _setup_dbus(self)
	# \brief Sets up the dbus class attribute.
	def _setup_dbus(self):
		DBusGMainLoop( set_as_default=True )
		self.mdns = SMdns()
		self.mdns.browse(self.handle_resource_found, self.handle_resource_lost)

	#-----------------------gui_handlers Eventos de la ventana--------------------------#

	def habilita_ayuda(self):
		self.gui.cambiarNombre.setEnabled(True)
		self.gui.btnHelp.setEnabled(True)

	## \fn mandar_captura_servidor(self, nick, user_muc_jid)
	# \brief Envia mensajes al servidor.
	# \param nick nick del usuario que envia el mensaje.
	def mandar_captura_servidor(self, nick):
		ruta = os.path.join(DESKTOP_RUTA,nick.strip())
		if not os.path.exists(ruta):
			os.mkdir(ruta)
		if os.path.exists(DESKTOP_RUTA+'/'+nick.strip()+'/screenshot.jpg'):
			#transformacion de la imagen para pasarla como texto
			with open(DESKTOP_RUTA+'/'+nick.strip()+'/screenshot.jpg', 'rb') as imageFile:
				msg = "IMG"+"|"+nick.strip()+"|"+base64.b64encode(imageFile.read())
			if self.jid_sala != "" and self.jid != "":
				self.enviar_mensaje(msg)

	## \fn request_join_muc(self, muc_jid)
	# \brief Unirse a un MUC.
	# \param muc_jid identificador del MUC.
	def request_join_muc(self, muc_jid):
		self.jid_sala = muc_jid
		id_intr = 0
		resource = self.resource_list[muc_jid]
		self.ip_muc = resource['ip']
		
		try:
			ip = get_ip()
		except NoDefaultDevice, e:
			lerror("AppCoreAlumno -> request_join_muc: %s" % e)
		
		if ip != self.ip_muc:
			if len(get_user_name()) == 0:
				logging.debug("El username: '%s'" % get_user_name())
				identify_user()
			else:
				id_intr = 1
			
			if id_intr == 1:
				#TODO: Quitar o cambiar el apaño (sufijo del jid y nick) para tener servidor y cliente
				#funcionando en la misma máquina:
				jid = 'a'+generate_jid(resource['hostname'])
				nick = ' ' + get_user_name()

				self.nick = nick
				self.jid = jid

				self.bot = ClienteAlumno(jid=jid, muc_jid=muc_jid, nick=nick, parent=self)
				self.bot.set_core_handlers(self.bot_handlers)
				#Handlers del modulo streaming
				self.bot.set_stream_handlers(self.stream_handlers)
				self.bot.connect((self.ip_muc, 5222))
				self.bot.process(threaded=True)
				self.log.info(QtGui.QApplication.translate("AppCoreAlumno.log", "Conectando al grupo: %1 - %2 (%3)",
														None, QtGui.QApplication.UnicodeUTF8).arg(
															resource['muc_name']).arg(
															resource['prof_name']).arg(
															resource['hostname'])
						)
		else:
			self.enviar_gui_msg('No puede conectarse a su propio grupo.','information')

	## \fn request_exit_muc(self)
	# \brief Handle desconectar.
	def request_exit_muc(self):
		try:
			resource = self.resource_list[self.resource_name]
			self.log.info(QtGui.QApplication.translate("AppCoreAlumno.log", "Desconectando del grupo: %1 - %2",
													None, QtGui.QApplication.UnicodeUTF8).arg(
															   resource['muc_name']).arg(
															   resource['prof_name'])
						)
		except KeyError:
			pass
		self.resource_name = None
		self.shared_info=None
		self.borrar_carpeta_capturas()
		self.stop_vlc()
		if self.bot is not None:
			if self.bot.controldesktop.pid_lockscreen != None:
				parar_procesos_activos(self.bot.controldesktop.pid_lockscreen)
			else:
				parar_procesos_activos()
				parar_vlc()
			self.bot.disconnect()
		#self.bot = None
		self.log.info(QtGui.QApplication.translate("AppCoreAlumno.log", "Desconectado."))
		self.gui.emit(QtCore.SIGNAL("connectionLost"), self.resource_list, self.resource_name)
		self.gui.emit(QtCore.SIGNAL("hideSharedClient"))
		self.stop_hcolaborativa()
		if self.flag_borra_jid == False:
			self.jid_sala = ""

		
	def solicitar_ayuda_servidor(self):
		msg ="AYUDA |"+str(self.nick)
		self.enviar_mensaje(msg)

	## \fn enviar_escritorio_imagen(self)
	# \brief Metodo necesario para conectarse al socket del servidor y mandar la captura solicitada.
	def enviar_escritorio_imagen(self):
		#time.sleep(5)
		sCliente = socket()
		CONEXION = (self.ipServidor, 6030)
		sCliente.connect(CONEXION)
    
		while True:
			f = open("ImgEscritorio", "rb")
			content = f.read(1024)

			while content:
				# Enviar contenido.
				sCliente.send(content)
				content = f.read(1024)
        
			break
    
		# Se utiliza el caracter de código 1 para indicar
		# al cliente que ya se ha enviado todo el contenido.
		try:
			sCliente.send(chr(1))
		except TypeError:
			# Compatibilidad con Python 3.
			sCliente.send(bytes(chr(1), "utf-8"))
	    
		# Cerrar conexión y archivo.
		sCliente.close()
		f.close()
		msg ="ENVIO_ESCRITORIO"
		self.enviar_mensaje(msg)

	#-----------------------bot_handlers Eventos del bot--------------------------#

	## \fn conectarse_muc(self, muc_jid)
	# \brief Manejador para conectarse a eventos del muc.
	# \param muc_jid MUC identifier.
	def handle_connected(self, muc_jid):
		if muc_jid in self.resource_list.keys():
			self.resource_name = muc_jid
			# Update Gui from others threads
			self.gui.emit(QtCore.SIGNAL("connectionStablished"), self.resource_list, self.resource_name)
			self.request_shared_info()
			self.log.info(QtGui.QApplication.translate("AppCoreAlumno.log", "Conexión establecida.",
													None, QtGui.QApplication.UnicodeUTF8))
		else:
			self.log.info(QtGui.QApplication.translate("AppCoreAlumno.log", "No se puede conectar al recurso %1, no \
																	esta en la lista.", None,
																	QtGui.QApplication.UnicodeUTF8).arg(muc_jid))
			self.request_exit_muc()

	## \fn handle_disconnected(self)
	# \brief Handle disconnected.
	def handle_disconnected(self):
		self.bot = None
		self.resource_name = None
		self.log.info(QtGui.QApplication.translate("SCoreAlumno.log", "Desconectado."))
		self.gui.emit(QtCore.SIGNAL("connectionLost"), self.resource_list, self.resource_name)

	def actualizar_nom_user_server(self):
		try:
			jid_sala = self.jid_sala
			self.request_exit_muc()
			self.request_join_muc(jid_sala)
		except KeyError, m:
			lerror("Error Actualizar Nombre: %s" % m)


	## \fn mensaje_estado_muc(self, code, msg)
	# \brief Handle xmpp bot errors, and emits signals to notify the GUI.
	# \param code Error code.
	# \param msg Error message.
	def mensaje_estado_muc(self, code, msg):
		# Show messages from others threads in the Gui
		self.log.warn(QtGui.QApplication.translate("AppCoreAlumno.log", "Estado %1: %2", None,
											QtGui.QApplication.UnicodeUTF8).arg(code).arg(msg))
		self.enviar_gui_msg(msg,'information')
		if int(code) in EXIT_CODE_MESSAGES:
			self.request_exit_muc()

	#metodo para el envio de mensajes informativos al usuario cliente
	def enviar_gui_msg(self, msg, tipo):
		self.gui.emit(QtCore.SIGNAL("msgReceived"), msg, tipo)

	## \fn realiza_captura(self, nick, user_muc_jid)
	# \brief Emite la señal para que la interfaz realice la captura de pantalla.
	# \param nick nick del usuario que envia el mensaje.
	# \param user_muc_jid jid del usuario que envia el mensaje
	def realiza_captura(self, nick, user_muc_jid):
		self.gui.emit(QtCore.SIGNAL("realizaCaptura"), nick, user_muc_jid)

	def enviar_mensaje(self, msg):
		self.bot.mensajeServidor(self.jid_sala, self.jid, msg)

	## \fn habilita_hcolaborativa(self, accion)
	# \brief Emite la señal para que la interfaz de usuario habilite el boton de la herramienta colaborativa.
	# \param accion la accion que se realiza, habilitar o deshabilitar el boton
	# \param nombreDocu, la direccion ip del servidor junto con el nombre del documento creado
	def stop_hcolaborativa(self):
		self.gui.emit(QtCore.SIGNAL("stop_hcolaborativa"))

	## \fn def reload_url_hc(self, url)
	# \brief Funcionalidad Documento colaborativo: emite la señal para actualizar el documento en la ventana.
	# \param url recibe la url del documento activo en el servidor
	def reload_url_hc(self, url):
		self.gui.emit(QtCore.SIGNAL("reloadURL"), url)

	## \fn def estadoHC_server(self, activar)
	# \brief Funcionalidad Documento colaborativo: Getiona si la funcionalidad debe estar activa o no.
	# \brief Si debe estar activa realizará la petición al servidor para realizar reload.
	# \param url recibe la url del documento activo en el servidor
	def estadoHC_server(self, activar):
		if activar == False and self.gui.HCStatus == True:
			self.stop_hcolaborativa()
		elif activar == True and self.gui.HCStatus == False:
			if self.resource_name != None:
				msg = "HC|"+str(self.nick)
				time.sleep(2)
				self.enviar_mensaje(msg)

	def emision_vlc(self):
		self.vlc_activo = True

	def stop_vlc(self):
		self.vlc_activo = False

	def realizar_imagen_escritorio(self, ipServidor):
		self.ipServidor = ipServidor
		self.gui.emit(QtCore.SIGNAL("escritorio"), self.ipServidor)


	#-----------------------FIN -- bot_handlers Eventos del bot--------------------------#


	## \fn request_shared_info(self)
	# \brief Make a RPC call for request teacher's shared information.
	# \brief Realiza una llamada RPC para la información compartida del profesor.
	def request_shared_info(self):
		self.log.info(QtGui.QApplication.translate("AppCoreAlumno.log", "Solicitando información de compartido.",
												None, QtGui.QApplication.UnicodeUTF8))
		prof_jid = self.resource_list[self.resource_name]["prof_jid"]
		self.bot.rpc_call(prof_jid, "SHARED_INFO", handler=self._set_shared_info)


	## \fn _set_shared_info(self)
	# \brief Receive and process the teacher's shared information.
	# \param *args RPC result params.
	# \brief Recibe y procesa la información compartida del maestro.
	def _set_shared_info(self, id, info):
		self.log.info(QtGui.QApplication.translate("AppCoreAlumno.log", "Recibida información de compartido.",
												None, QtGui.QApplication.UnicodeUTF8))
		self.shared_info = dict(info)
		self.shared_info["nick"] = self.bot.nick
		self.shared_info["group"] = self.resource_list[self.resource_name]["muc_name"]


	def get_shared_info(self):
		return self.shared_info

	## \fn borrar_carpeta_capturas(self)
	# \brief Cuando el usuario se desconecta del grupo borra su carpeta de almacenamiento de capturas.
	def borrar_carpeta_capturas(self):
		if self.nick != None:
			ruta = os.path.join(DESKTOP_RUTA,self.nick.strip())
			if os.path.exists(ruta):
				sentencia = "rm -r '"+DESKTOP_RUTA+'/'+str(self.nick.strip())+"'"
				os.system(sentencia)

	## \fn handle_resource_found(self, name, domain, address, port, info_dict)
	# \brief Handle the Resource Found event.
	# \param name Resource Name.
	# \param domain Resource Domain.
	# \param address Resource Address.
	# \param port Resource Port.
	# \param info_dict Aditional information necessary for configure resource.
	def handle_resource_found(self, name, domain, address, port, info_dict):
		self.lista=name
		info_keys = info_dict.keys()

		if 'muc_jid' in info_keys and 'muc_name' in info_keys and 'prof_name' in info_keys and 'prof_jid' in info_keys:
			info_dict['service_name'] = name
			with self.resource_list_lock:
				self.resource_list[info_dict['muc_jid']] = info_dict
		if self.jid_sala != "":
			if ping(self.ip_muc):
				self.estadoHC_server(True)
		self._update_gui()

	## \fn handle_resource_lost(self, *args)
	# \brief Handle the Resource Lost event.
	# \param *args Array with information about resource lost.
	def handle_resource_lost(self, *args):
		resource_index = None
		logging.info('Resource Lost: %s' % args[0])
		for resource in self.resource_list.values():
			if resource['service_name'] == args[0]:
				resource_index = resource['muc_jid']
				continue
		if resource_index is not None:
			resource = self.resource_list[resource_index]
			self.log.info(QtGui.QApplication.translate("AppCoreAlumno.log", "Grupo perdido: %1 - %2",
													None, QtGui.QApplication.UnicodeUTF8).arg(
															   resource['muc_name']).arg(
															   resource['prof_name'])
						)
			if resource['muc_jid']==self.resource_name:
				if ping(self.ip_muc):
					self.enviar_gui_msg('El grupo ha sido eliminado.','information')
					self.request_exit_muc()
					
			with self.resource_list_lock:
				del self.resource_list[resource_index]
			if self.gui.HCStatus == True:
				if not ping(self.ip_muc):
					self.estadoHC_server(False)
			self._update_gui()

	## \fn update_gui(self)
	# \brief Sets up the GUI from the variables resource_list and resource_name
	def _update_gui(self):
		logging.info('Resource List: \n\t%s' % self.resource_list)
		self.gui.update_gui(self.resource_list, self.resource_name)

	def borrar_desktop(self):
		result_listado = commands.getstatusoutput('ls '+DESKTOP_RUTA)
		if result_listado[1] != "":
			os.system('rm -r '+DESKTOP_RUTA+'/*')

	## \fn exit_application(self)
	# \brief Make things when client closed..
	def exit_application(self):
		self.request_exit_muc()
		self.log.info(QtGui.QApplication.translate("AppCoreAlumno.log", "Cerrando cliente."))
		self.borrar_desktop()

		self.appcore_handlers["EXIT"]()

if __name__=='__main__':
#    import sys
#    reload(sys)
#    sys.setdefaultencoding("utf-8")
    sc = AppCoreAlumno()
    sc.run()
