# -*- coding: utf-8 -*-
## \namespace alumno
import logging
import os
import thread
from PyQt4 import QtGui

from hga.controlgrupos.cliente.settingscliente import RPC_MODULES
from hga.controlgrupos.cliente.settingscliente import CODE_MESSAGES
from hga.controlgrupos.common.exception import NoDefaultDevice
from hga.controlgrupos.common.settings import HOMEDIR
from hga.controlgrupos.common.settings import LOGGING_CONFIG_FILE

from hga.controldesktop.controlpeticiones import ControlProcesos

from hga.controlgrupos.common.sbotxmpp import SBotXMPP

from hga.controlgrupos.common.user_info import identify_user
from hga.controlgrupos.common.user_info import get_ip

logging.config.fileConfig(LOGGING_CONFIG_FILE)
linfo = logging.info
lerror = logging.error
ldebug = logging.debug

## \class ClienteAlumno(SBotXMPP)
#  \brief Crea el bot en HGR para el cliente. 
class ClienteAlumno(SBotXMPP):
    ## \fn __init__(self, jid, muc_jid, nick, passwd="anonymous") 
    #  \brief Nuevo bot para el cliente.
    # \param jid JID del Bot. (ej a901b0e5b7f06@pc141-1.local/HGR)
    # \param muc_jid JID del MUC(sala) para unirse. (ej gdfgdgdfg@conference.pc141-1.local)
    # \param nick El nick del cliente que se usara en la sala.
    # \param passwd Password for the xmpp server client. Not used.
    def __init__(self, jid, muc_jid, nick, passwd="anonymous", parent=None):
        self.RPC_MODULES = RPC_MODULES
        super(ClienteAlumno, self).__init__(jid, muc_jid, nick, passwd, parent)
        self.controldesktop = ControlProcesos()

    ## \fn set_core_handlers(self, handlers) 
    #  \brief Establece los manejadores que usa el appCore
    # \param handlers Dictionary in the form EVENT:method
    def set_core_handlers(self, handlers):
        self.core_handlers = handlers

    #Manejadores del modulo controlstreming
    def set_stream_handlers(self, handlers):
        self.stream_handlers = handlers

    ## \fn  _load_event_handlers(self)
    #  \brief  Busqueda de eventos del handler: groupchat_presence
    def _load_event_handlers(self):
        # Cuando se añada un nuevo evento del handler, dejamos que el padre cargue sus propios controladores
        super(ClienteAlumno, self)._load_event_handlers()
        self.add_event_handler("groupchat_presence", self.handle_nuevo_muc)
        self.add_event_handler("groupchat_message", self.handle_message_incomming)


    ## \fn  handle_nuevo_muc(self, entry)
    # \brief  Maneja la presencia del MUC.
    # \param entry XMPP presence stanza
    def handle_nuevo_muc(self, entry):
        user = entry.find('{%(ns)s}x/{%(ns)s}item' %
            {'ns': 'http://jabber.org/protocol/muc#user'})
        user_muc_jid = entry.getFrom()
        muc_jid = user_muc_jid.bare
        nick = user_muc_jid.resource
        if nick == self.nick:
            tipo = entry.getType()
            code = None
            if tipo == "unavailable":
                # Cuando se nos hecha del muc o se nos banea
                status = entry.find('{%(ns)s}x/{%(ns)s}status' %
                            {'ns': 'http://jabber.org/protocol/muc#user'})
                if status is not None:
                    code = status.get("code")
                else:
                    self.stream_handlers["KILL_VLC"]()
                    code = 7337 #Se destruye la sala
            elif tipo == "error":
                error = entry.find('{jabber:client}error')
                code = error.get("code")
            elif tipo == "available":
                #linfo("Inform the core about our own presence.")
                self.core_handlers["STATUS_CONNECTED"](muc_jid)
            else:
                lerror("Unhandled presence type: '%s'" % tipo)
            # Si tenemos un código, algo raro ha sucedido . manejarlo (por ejemplo el grupo esta cerrado 407)
            if code:
                self.handle_estado_usuario(code)

    ## \fn def mensajeServidor(self, jid_sala, jid, msg)
    # \brief Envia el mensaje al servidor por parte del cliente
    # \param jid_sala Jid de la sala conectada
    # \param jid Jid del cliente
    # \param msg Mensaje que recibe el 
    # \param ip_muc IP del MUC
    def mensajeServidor(self, jid_sala, jid, msg):
        try:
            get_ip() # Si no hay conexión NoDefaultDevice
            self.sendMessage(mto=jid_sala, mbody=msg, mtype='groupchat', mfrom=jid)
        except KeyError, e:
            lerror("ServerProfesor -> sendmessage: %s" % e)
        except NoDefaultDevice, e:
            lerror("ServerProfesor -> mensajeAlumno: %s" % e)

    ## \fn def handle_message_incomming(self, msg):
    # \brief Funcion para recibir mensajes
    # \param msg Mensaje que recibe el usuario
    def handle_message_incomming(self, msg):
        #user_muc_jid = msg.getFrom()
        #muc_jid = user_muc_jid.bare
        #nick = user_muc_jid.resource
        cuerpoMsg = msg['body']
        if cuerpoMsg.startswith('screenshoot'):
            listCommand = cuerpoMsg.split("/")
            self.core_handlers["CAPTURA"](listCommand[1],listCommand[2])
     
        #Se realiza el cambio de nombre del usuario si ha sido solicitado.
        if cuerpoMsg.startswith("IDENTIFY"):
            user_msg = cuerpoMsg.split("|")
            identify_user(user_msg[1])
            self.core_handlers["ACTUALIZAR_NOM_USER_SERVER"]()
        # Se filtra en primer lugar por módulo para evitar que se realice la comprobación
        # en caso de que sean acciones no relacionadas con controldesktop
        if cuerpoMsg.startswith("DESKTOP"):
            user_msg = cuerpoMsg.split("|")
            result_msg = self.controldesktop.ejecutar_peticion(user_msg, self.nick)
            if result_msg != None:
                if result_msg.startswith("DESKTOP"):
                    self.core_handlers["ENVIAR_MENSAJE"](result_msg)

                elif result_msg.startswith("CLIENT"):
                    result_msg = result_msg.split("|")
                    self.core_handlers["SEND_GUI_MSG"](result_msg[1],result_msg[2])
        
        if cuerpoMsg.startswith("vlc"):
                listCommand = cuerpoMsg.split("|")
                ip_servidor = listCommand[1]
                self.stream_handlers["RECIBIR_EMISION"](ip_servidor)
                self.core_handlers["EMISION_VLC"]()

        if cuerpoMsg.startswith("kill"):
            self.stream_handlers["KILL_VLC"]()
            self.core_handlers["STOP_VLC"]()

        if cuerpoMsg.startswith("START_EDITOR"):
            listCommand = cuerpoMsg.split("|")
            path = os.path.join(HOMEDIR,"Documentos")
            os.chdir(os.path.abspath(path))
            self.core_handlers["RELOAD_URL_HC"](listCommand[1])
        
        if cuerpoMsg.startswith("STOP_EDITOR"):
            os.chdir(os.path.abspath("/usr/bin"))
            self.core_handlers["STOP_HCOLABORATIVA"]()

        if cuerpoMsg.startswith("AYUDA-ACTIVA"):
            self.core_handlers["HABILITA_AYUDA"]()

        if cuerpoMsg.startswith("ESCRITORIO"):
            listCommand = cuerpoMsg.split("|")
            ipServidor = listCommand[2]
            self.core_handlers["REALIZAR_IMAGEN_ESCRITORIO"](ipServidor)


    ## \fn handle_estado_usuario(self, code)
    #  \brief  Maneja diferentes errores o codigos
    #  \param[in] Message code number
    def handle_estado_usuario(self, code):
        #linfo('Informacion acerca del estado del usuario\n%s\n', ('-'*80))
        # Informacion del core sobre baneado, kick, etc.
        if code == '110':
            pass
        else:
            try:
                # Obtiene la descripcion del codigo
                msg = CODE_MESSAGES[int(code)]
            except KeyError, m:
                lerror("Unknown error code: %s" % code)
            else:
                #linfo(msg)
                self.core_handlers["STATUS_MESSAGE"](code, msg)
            
    ## \fn _handle_desconectar(self, event)
    # \brief Maneja el evento desconexion.
    # \param event XMPP stanza.
    def _handle_disconnected(self, event):
        # Just let the core handle the disconnection event.
        self.core_handlers["STATUS_DISCONNECTED"]()
        
        
if __name__=="__main__":
    import sys
    import getopt
    import base64
    print 'LLEGO AL cliente'
    opts = dict(getopt.getopt(sys.argv[1:], "u:n:")[0])
    user = opts.get("-u", "")
    nick = opts.get("-n", "Nick del alumno")
    muc = opts.get("-m", "salita")

    host = "PCPRUEBA"
    jid = "%s@%s" % (user, host)
    password = "alumno"
    muc_jid = "%s@conference.%s" % (muc, host)
    print "Conectado:"
    print "JID: %s" % jid
    print "Nick: %s" % nick
    bot = ClienteAlumno(jid, muc_jid, nick, passwd=password)
    bot.connect()
    bot.process(threaded=False)