#!/bin/env python
# -*- coding: UTF-8 -*-

import os
import re
import commands
import threading
import time
import logging

from PyQt4 import QtGui, QtCore

from hga.controlgrupos.common.user_info import get_ip
from hga.controlgrupos.common.settings import HOMEDIR
from hga.controlgrupos.common.settings import LOGGING_CONFIG_FILE
from hga.controlgrupos.common.exception import NoDefaultDevice


FILTROS_STREAMING = "Archivos Multimedia(*.mkv *.mpg *.wmv *.mp4 *.avi *.mpeg *.wma *.mp3 *.cda *.aac *.ogg *.wav)"

logging.config.fileConfig(LOGGING_CONFIG_FILE)
lerror = logging.error

##\class stream_Servidor
# \brief Clase del modulo controlStreaming para realizar la emision de videos y audios por parte del servidor
class stream_Servidor:

	def __init__(self, coreProfesor):
		self.core = coreProfesor
		self.ip_servidor = self.obtenerIP()
		self.cliente_streaming = None
		self.cadena = None

	## \fn obtenerIP(self)
	# \brief Obtiene la direccion ip del servidor para realizar la emision.
	# \return ip_servidor
	def obtenerIP(self):
		try:
			ips = get_ip()
			ip = ips.split(".")
			self.ip_servidor = "227.255"+"."+ip[2]+"."+ip[3]
		except NoDefaultDevice, e:
			lerror("Petición -> stream_server: obtenerIp. No hay conexión: %s" % e)
			
	## \fn tratar_fuente(self, tipo)
	# \brief Detecta y trata las fuentes del tipo cd y dvd
	# \param tipo, el tipo de fuente cd o dvd
	def tratar_fuente(self, tipo):
		fuente = ''
		dispositivos = self.buscar_dispositivos()

		if tipo == 'dvd':
			patron = 'data disc type 1'
			msg = u"No es posible emitir DVD. Es probable que su equipo informático no disponga de unidad lectora de discos DVD o el disco introducido en el lector no se corresponda con un DVD de vídeo. En cualquier caso, revise su configuración y vuelva a intentar la emisión."
			cabeceraFuente = "dvdsimple:///dev/"
		else:	
			patron = 'audio disc'
			msg = u"No es posible emitir CD. Es probable que su equipo informático no disponga de unidad lectora de CD o el disco introducido en el lector no se corresponde con un CD de audio. En cualquier caso, revise su configuración y vuelva a intentar la emisión."
			cabeceraFuente = "cdda:///dev/"

		for disp in dispositivos:
			if disp != '' and disp != 'drive name:':
				info_cd_buffer = commands.getoutput('setcd -i /dev/%s' % disp)
				info_cd = info_cd_buffer.split('\n')
				pattern = re.compile(patron)
				for linea in info_cd:
					if re.search(pattern, linea) != None:
						fuente = cabeceraFuente + disp
						return fuente
		self.core.enviar_gui_msg(msg, "information")

	## \fn buscar_dispositivos(self)
	# \brief Realiza una busqueda en el sistema para detectar los dispositivos en el equipo
	def buscar_dispositivos(self):
		if os.access('/proc/sys/dev/cdrom/info', os.F_OK) == True:
			info_lectores_buffer = commands.getoutput('cat /proc/sys/dev/cdrom/info')
			info_lectores = info_lectores_buffer.split('\n')
			for linea in info_lectores:
				pattern = re.compile('drive name')
				if re.search(pattern, linea) != None:
					dispositivos = linea.split('\t')
					break
		else:
			dispositivos = ''
		return dispositivos

	## \fn ejecutar_emision(self, fuente_streaming, alumno)
	# \brief Metodo encargado de realizar la emision, obtiene la fuente de emision y diferencia si es emision grupal o individual
	# \param fuente_streaming, desde donde selecciona el usuario que quiere emitir (fichero, cd o dvd)
	# \param alumno, si es un alumno o el grupo
	def ejecutar_emision(self, fileStreaming, alumno):
		if self.ip_servidor == None:
			self.obtenerIP()
		self.cliente_streaming = alumno

		if fileStreaming != None and fileStreaming != "":
			command = "netstat -np 2>/dev/null| grep :8080 | awk '{print $7}' | cut -d ""/"" -f 1"
			self.tipo_streaming(fileStreaming)
			self.crear_streaming()
			self.comunicaciones_cliente('emision')
			if alumno != None:
				alumno = self.cliente_streaming[0]
			self.core.activa_usuarios_stream(alumno)

	## \fn ejecuta_streaming_server(self, cadena)
	# \brief Emite el video o audio seleccionado
	# \param cadena, la sentencia a ejecutar vlc
	def ejecuta_streaming_server(self):
		os.system(str(self.cadena))
		time.sleep(5)
		self.comunicaciones_cliente('kill')

	## \fn tipo_streaming(self, fuente)
	# \brief Filtra el tipo de fuente
	# \param fuente, la fuente seleccionada por el usuario (cd, dvd o archivo)
	def tipo_streaming(self, fuente):
		if str(fuente).startswith('cdda:'):
			self.gestion_audio(fuente)
		elif str(fuente).startswith('dvdsimple:'):
			self.gestion_video(fuente)
		else:
			array = fuente.split('/')
			nombre = array[len(array)-1]
			extension = nombre.split('.')
			extTipo = extension[1]
			audios = ['wma','mp3','cda','aac','ogg','wav']
			if extTipo in audios:
				self.gestion_audio(fuente)
			else:
				self.gestion_video(fuente)

	## \fn crear_streaming(self, fuente)
	# \brief Dependiendo de la fuente que le llega crea la sentencia necesaria para emitir
	# \param fuente, tipo de fuente seleccionada para la emision
	def crear_streaming(self):
		if self.cadena != None:
			thread = threading.Thread(target=self.ejecuta_streaming_server, args=())
			thread.start()

	## \fn gestion_audio(self, fuente)
	# \brief Crea la cadena de audio a reproducir dependiendo de la fuente
	# \param fuente, tipo de archivo a ejecutar
	def gestion_audio(self, fuente):
		if str(fuente).endswith('ogg'):
			self.cadena = "vlc '"+fuente+"' '--quiet' '--play-and-exit' '--sout' '#transcode{vcodec=mp2v,vb=800,acodec=mpga,ab=128,channels=2,samplerate=44100}:duplicate{dst=rtp{dst=%s,port=8080,mux=ts},dst=display}'" % self.ip_servidor
		else:
			self.cadena = "vlc '"+fuente+"' '--quiet' '--play-and-exit' '--sout' '#transcode{vcodec=h264,acodec=mpga,ab=128,channels=2,samplerate=44100}:duplicate{dst=rtp{dst=%s,port=8080,mux=ts},dst=display}'" % self.ip_servidor

	## \fn gestion_video(self, fuente)
	# \brief Crea la cadena de video a reproducir dependiendo de la fuente
	# \param fuente, tipo de archivo a ejecutar
	def gestion_video(self, fuente):
		if str(fuente).endswith('mkv'):
			self.cadena = "vlc '"+fuente+"' '--quiet' '--play-and-exit' '--sout' '#duplicate{dst=rtp{dst=%s,port=8080,mux=ts,ttl=2},dst=display}'" % self.ip_servidor
		elif str(fuente).startswith('dvdsimple:'):
			self.cadena = "vlc '"+fuente+"' '--quiet' '--play-and-exit' '--sout' '#transcode{vcodec=mp4v,scale=1,channels=2}:duplicate{dst=rtp{dst=%s,port=8080,mux=ts},dst=display}'" % self.ip_servidor
		else:
			self.cadena = "vlc '"+fuente+"' '--quiet' '--play-and-exit' '--sout' '#transcode{vcodec=mp4v,acodec=mpga,vb=800,scale=1,ab=128,channels=2,samplerate=44100}:duplicate{dst=rtp{dst=%s,port=8080,mux=ts},dst=display}'" % self.ip_servidor

	## \fn comunicaciones_cliente(self, tipo)
	# \brief Envia mensaje al cliente para comunicarle la emision o la finalizacion del streaming
	# \param tipo, tipo de mensaje a enviar al cliente
	def comunicaciones_cliente(self, tipo):
		if tipo == 'kill':
			msg = "kill"
			self.core.detiene_objeto_stream()
		elif tipo == 'emision':
			msg = "vlc|"+self.ip_servidor

		if self.cliente_streaming == None:
			for alumno in self.core.users:
				self.core.sendmessage(alumno, msg)
		else:
			alumnoUnico = self.cliente_streaming[0]
			if self.busqueda_alumno(alumnoUnico):
				self.core.sendmessage(alumnoUnico, msg)

	def busqueda_alumno(self, alumnoUnico):
		existe = False
		if alumnoUnico in self.core.users:
			existe = True
		return existe