#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
from PyQt4.QtCore import *
from PyQt4.QtGui import *

from hga.gui.servidor.sgrupos_red import SGruposRed
from hga.gui.servidor.sbaneados import SBaneados

class GestionControlGrupos:
	def __init__(self, gui, core, stream=None):
		self.gui = gui
		self.core = core
		self.stream = stream

####### DEFINICION DE LOS HANDLERS

	def gui_handlers_servidor(self):
		#Diccionario con los manejadores de las peticiones del GUI
		self.gui_handlers = {
			"CAMBIAR_NOMBRE_SERVER": self.cambiar_nombre_server,
			"CREATE_RESOURCE_CB": self.crear_recurso,
			"DESTROY_RESOURCE_CB": self.destruir_recurso,
			"KICK_USER_CB": self.expulsar_alumno,
			"BAN_USER_CB": self.banear_alumno,
			"UNBAN_USER_CB": self.desbanear_alumno,
			"IDENTIFY_USER_CB": self.identificar_alumno,
			"OPEN_MUC_CB": self.abrir_muc,
			"CLOSE_MUC_CB": self.cerrar_muc,
			"CLOSE_EVENT_CB": self.cerrarEvento,
			"ORDER_NAME":self.ordenar_nombre,
			"ENVIAR_MENSAJE":self.envio_mensajes,
			"VIEW_RESOURCES":self.view_resources,
			"EMITIR_STREAMING":self.emitir_streaming,
			"TRATAR_FUENTE":self.tratar_fuente,
			"DELAY_CAPTURAS":self.get_delay_capturas,
			"SHARED_CB": self.shared_cb,
			"GET_SHARED_INFO_SERVER": self.obtener_info_compartida_server,
			"HABILITA_AYUDA_USER": self.habilita_ayuda_user,
		}
		return self.gui_handlers
	
	def gui_handlers_cliente(self):
		self.gui_handlers = {
			"REQUEST_JOIN_MUC": self.request_join_muc,
			"REQUEST_EXIT_MUC": self.request_exit_muc,
			"GET_SHARED_INFO": self.obtener_info_compartida,
			"EXIT_APPLICATION": self.exit_application,
			"RECONNECT": self.reconectar,
			"ENVIAR_CAPTURA_SERVIDOR": self.enviar_captura_servidor,
			"ACTUALIZAR_NOM_USER_CLIENT": self.actualizar_nom_user_client,
			"SOLICITAR_AYUDA_SERVIDOR": self.solicitar_ayuda_servidor,
			"ENVIAR_ESCRITORIO_IMAGEN": self.enviar_escritorio_imagen,
		}
		return self.gui_handlers


		
	#----------EVENTOS DEL SERVIDOR-----------#
	def cambiar_nombre_server(self):
		self.core.cambiar_nombre_server()

	def envio_mensajes(self,nick, msg):
		self.core.sendmessage(nick, msg)
		
	def crear_recurso(self,nombre_muc):
		self.core.crear_recurso(nombre_muc)

	def destruir_recurso(self):
		self.core.destruir_recurso()
		self.gui.update_gui(self.core.users)

	def abrir_muc(self):
		self.core.abrir_muc()

	def cerrar_muc(self):
		self.core.cerrar_muc()

	def expulsar_alumno(self, nick):
		self.core.expulsar_alumno(nick)
		self.gui.update_gui(self.core.users)

	def banear_alumno(self, nick):
		self.core.banear_alumno(nick)
		self.gui.update_gui(self.core.users)

	def desbanear_alumno(self, nick):
		self.core.desbanear_alumno(nick)
		self.gui.update_gui(self.core.users)
		
	def identificar_alumno(self, nick, username):
		self.core.identificar_alumno(nick, username)
		self.gui.update_gui(self.core.users)

	def ordenar_nombre(self):
		self.gui.update_gui(self.core.users)

	def habilita_ayuda_user(self, nick):
		self.core.habilita_ayuda_user(nick)

	def cerrarEvento(self):
		self.core.cerrarEvento()
		try:
			self.gui.log_win.hide()
		except:
			pass
		self.gui.hide()

	## \fn emitir_streaming(self, fuenteStreaming, alumnoUnico=None)
	# \brief Transmite la señal al servidor streaming para ejecutar desde la fuente seleccionada.
	# \param fuenteStreaming, fuente seleccionada por el usuario desde donde quiere emitir (fichero, cd o dvd)
	# \param alumnoUnico, distingue si se quiere emitir al grupo o a un cliente en particular
	def emitir_streaming(self, fuenteStreaming, alumnoUnico=None):
		self.stream.ejecutar_emision(fuenteStreaming, alumnoUnico)

	def tratar_fuente(self, fuenteStreaming):
		fileStreaming = self.stream.tratar_fuente(fuenteStreaming)
		return fileStreaming

	## \fn get_delay_capturas(self, delay)
	# \brief Obtiene el delay de el usuario ha seleccionado para cambiar el tiempo entre capturas.
	# \param delay, el tiempo seleccionado
	def get_delay_capturas(self, delay):
		self.gui.delay_modificado = delay #guardo en un atributo del sgestion el valor seleccionado
		self.core.set_delay(self.gui.delay_modificado)

	def view_resources(self):
		self.gui_red = SGruposRed(self.core)
		if self.gui_red is not None:
			self.gui_red.show()


	def shared_cb(self, estado):
		self.core.share_dir(estado)


	def obtener_info_compartida_server(self):
		shared_info = dict()
		shared_info['IP'] = self.core.shared.IP
		shared_info['port'] = self.core.shared.port
		shared_info['user'] = self.core.shared.user
		shared_info['password'] = self.core.shared.password
		shared_info['group'] = self.core.shared.group
		shared_info['nick'] = self.core.nick
		return shared_info

	#----------EVENTOS DEL CLIENTE-----------#
	def enviar_captura_servidor(self,nick):
		self.core.mandar_captura_servidor(nick)

	def enviar_escritorio_imagen(self):
		self.core.enviar_escritorio_imagen()

	def request_join_muc(self, muc_jid):
		self.core.request_join_muc(muc_jid)

	def request_exit_muc(self):
		self.core.request_exit_muc()

	def obtener_info_compartida(self):
		shared_info = self.core.get_shared_info()
		return shared_info

	def exit_application(self):
		self.core.exit_application()

	def reconectar(self):
		self.core.reconectar()

	def actualizar_nom_user_client(self):
		self.core.actualizar_nom_user_client()

	def solicitar_ayuda_servidor(self):
		self.core.solicitar_ayuda_servidor()
