# -*- coding: utf-8 -*-
##\namespace alumno
import os
import sys
import logging

from PyQt4 import QtGui, QtCore

from hga.controlcompartir.cliente.sshareddirclient import SSharedDirClient
from hga.controlgrupos.common.settings import DESKTOP_RUTA
from hga.controldesktop.gui_controldesktop.ventana_msg_info_client import VentanaMensajeInfoClient
from hga.controleditor.gui_controleditor.scontroleditor_cliente import sControlEditor_cliente


#from hga.gui.common.slog import SLog

from hga.gui.alumno.ui_sguialumno import Ui_SGuiAlumno

import hga.gui.common.icon_rc


##\class ResourceItem
# \brief A Custom List Widget Item. Inherited from QT List Widget Item.
class ResourceItem(QtGui.QListWidgetItem):
    muc_jid = None

    ## \fn setEnabled(self, enabled=True)
    # \brief Update QListWidgetItem Flgas to enable/disable element.
    # \param enabled Boolean, indicates enable/disable state to set.
    def setEnabled(self, enabled=True):
        if enabled:
            self.setFlags(QtCore.Qt.NoItemFlags | QtCore.Qt.ItemIsEnabled)
        else:
            self.setFlags(QtCore.Qt.NoItemFlags)

    ## \fn isEnabled(self)
    # \brief Return boolean value indicates if item is enabled or disabled.
    def isEnabled(self):
        return (self.flags() == QtCore.Qt.ItemIsEnabled)


##\class SGuiAlumno
# \brief The student's GUI. Inherited from QT main window and the user UI.
class SGuiAlumno(QtGui.QMainWindow, Ui_SGuiAlumno):

    ## \fn __init__(self)
    # \brief GUI constructor, sets up the GUI and other things.
    def __init__(self):
        super(SGuiAlumno, self).__init__(None)
        self.setupUi(self)
        self._create_sysTrayIcon()
        self.HCStatus = False
        self.connect_signals()

        # Posición inicial de la ventana.
        self._center()
        self._position = self.pos()
        self.status = True
        self.gui_cEditor_cliente = sControlEditor_cliente(self)
        self.gui_cEditor_cliente.controleditor_signals()

        # Shared dir client window
        self.shared_win=None

    ## \fn set_core_handlers(self, handlers)
    # \brief Set core dict handlers.
    # \param handlers Handlers dictionary in the form EVENT:method
    # \param parent Parent of the GUI
    def set_core_handlers(self, handlers):
        self.core_handlers = handlers

    ## \fn connect_signals(self)
    # \brief Connects widgets' signals with different methods.
    def connect_signals(self):
        self.connect(self.connectButton, QtCore.SIGNAL("toggled(bool)"), self.realizar_conexion_cs)
        self.connect(self.ResourceListWidget, QtCore.SIGNAL("itemActivated(QListWidgetItem *)"), self._item_activated)
        self.connect(self.actionAcercaDe, QtCore.SIGNAL("triggered()"), self.show_about)
        QtCore.QObject.connect(self.btnFCompartidos, QtCore.SIGNAL("clicked()"), self.show_shared)
        #self.connect(self.actionLog, QtCore.SIGNAL("triggered()"), self.show_log)
        self.connect(self.cambiarNombre, QtCore.SIGNAL("triggered()"), self.cambiarNombreCliente)
        self.connect(self.btnHelp, QtCore.SIGNAL("clicked()"), self.solicitarAyuda)
        
        # Signals from others threads (Qt.AutoConnection / Qt.QueuedConnection):
        self.connect(self, QtCore.SIGNAL("realizaCaptura"), self.hacer_captura)
        self.connect(self, QtCore.SIGNAL("msgReceived"), self._message, QtCore.Qt.QueuedConnection)
        self.connect(self, QtCore.SIGNAL("connectionStablished"), self.update_gui, QtCore.Qt.QueuedConnection)
        self.connect(self, QtCore.SIGNAL("connectionLost"), self.update_gui, QtCore.Qt.QueuedConnection)
        self.connect(self, QtCore.SIGNAL("hideSharedClient"), self._hide_sharedclient, QtCore.Qt.QueuedConnection)
        self.connect(self, QtCore.SIGNAL("escritorio"), self.escritorio)
        #self.connect(self, QtCore.SIGNAL("triggered()"), self.closeEvent)


    ## \fn cambiarNombreCliente(self)
    # \brief Envia la peticion al appcorealumno para que cambie el nombre del cliente.
    def cambiarNombreCliente(self):
        self.core_handlers["ACTUALIZAR_NOM_USER_CLIENT"]()

    ## \fn hacer_captura(self, nick)
    # \brief Realiza la captura del escritorio del cliente.
    # \param nick nick del cliente
    def hacer_captura(self, nick):
        p = QtGui.QPixmap.grabWindow(QtGui.QApplication.desktop().winId())
        p = p.scaled(200,118,QtCore.Qt.IgnoreAspectRatio, QtCore.Qt.SmoothTransformation)
        if p != None:
            ruta = os.path.join(DESKTOP_RUTA,nick,"screenshot.jpg")
            p.save(ruta, 'jpg')
            self.core_handlers["ENVIAR_CAPTURA_SERVIDOR"](nick)

    ## \fn escritorio(self)
    # \brief Realiza la captura del escritorio del cliente puntual por peticion expresa del servidor.
    def escritorio(self):
        p = QtGui.QPixmap.grabWindow(QtGui.QApplication.desktop().winId())
        if p != None:
            p.save('ImgEscritorio', 'jpg')
            self.core_handlers["ENVIAR_ESCRITORIO_IMAGEN"]()

    ## \fn _center(self)
    # \brief Center main window on screen.
    def _center(self):
        screen = QtGui.QDesktopWidget().screenGeometry()
        size = self.geometry()
        self.move((screen.width()-size.width())/2, (screen.height()-size.height())/2)

    ## \fn _create_sysTrayIcon(self)
    # \brief Create the systray icon an their menu for exit aplication.
    def _create_sysTrayIcon(self):
        self.trayIcon = QtGui.QSystemTrayIcon(self)
        self.trayIcon.setIcon(QtGui.QIcon(":/common/icon/profile14.png"))
        self.trayIcon.show()

        self.context_menu = QtGui.QMenu()

        self.action_uniconize = self.context_menu.addAction(QtGui.QIcon(":/common/icon/magnifying40.png"), QtGui.QApplication.translate("SGuiAlumno", "&Ver grupos disponibles"))
        self.connect(self.action_uniconize, QtCore.SIGNAL("triggered(bool)"), self._trayIcon_uniconize)

        self.action_close = self.context_menu.addAction(QtGui.QIcon(":/common/icon/symbol21.png"), QtGui.QApplication.translate("SGuiAlumno", "&Salir"))
        self.connect(self.action_close, QtCore.SIGNAL("triggered(bool)"), self.close)
        
        self.trayIcon.setContextMenu(self.context_menu)

    ## \fn _show_about(self)
    # \brief Open a dialog with information about this program.
    def show_about(self):
        msg = QtCore.QString(u"""Herramienta de Gestión de Recursos CGA 2016""")
        self._message(msg,"about")


    def show_shared(self):
        shared_info = self.core_handlers["GET_SHARED_INFO"]()
        try:
            if shared_info:
                self.shared_win=SSharedDirClient(parent=None, data=shared_info)
                self.shared_win.setAttribute(QtCore.Qt.WA_DeleteOnClose)
                self.shared_win.show()
            else:
                self._message(u"<br>No estás conectado a ningún grupo.","critical")
        except:
            self._message(u"<br>Servicio de archivos compartidos desactivado.","critical")
            

    ## \fn _show_log(self)
    # \brief Open a event log's window..
    #def show_log(self):
        #self.log_win=SLog(logging.getLogger('hgr.client'))
        #self.log_win.show()


    ## \fn _trayIcon_show_hide(self)
    # \brief Toggle the application state (iconize/uniconize).
    def _trayIcon_uniconize(self):
        self.core_handlers["RECONNECT"]()
        if not self.isVisible():
            self.uniconize()
            self.showNormal()
            self.setFocus()

    ## \fn _hideEvent(self, event)
    # \brief Handle the hide event.
    # \param event Event.
    def hideEvent(self, event):
        if event.spontaneous():
            self.iconize()

    ## \fn iconize(self)
    # \brief Iconize the application.
    def iconize(self):
        self._position = self.pos()
        self.hide()

    ## \fn uniconize(self)
    # \brief Uniconize the application.
    def uniconize(self):
        if not self.isVisible():
            self.move(self._position)
            self.show()

    ## \fn update_gui(self)
    # \brief Update the gui apperance from variables resource_list and resource_name.
    # \param resource_list Contains the resource list to show en ListWidget.
    # \param resource_name Contaions the resource name wich we connected, in case we connected.
    def update_gui(self, resource_list, resource_name=None):
        self.ResourceListWidget.clear()
        self.trayIcon.setIcon(QtGui.QIcon(":/common/icon/profile14.png"))
        
        self.status = True
        self.actualizar_conexion()        

        for resource in resource_list.values():
            item = ResourceItem(resource['muc_name'])
            item.setToolTip(resource['prof_name'])
            item.muc_jid = resource['muc_jid']
            item.setIcon(QtGui.QIcon(":/common/icon/icon1.png"))

            if resource_name is not None:
                # This means has connected
                self.status = False
                self.actualizar_desconexion()
                item.setEnabled(item.muc_jid==resource_name)

                item.setIcon(QtGui.QIcon(":/common/icon/icon2.png"))
                self.trayIcon.setIcon(QtGui.QIcon(":/common/icon/user42.png"))

            self.ResourceListWidget.addItem(item)

    ## \fn actualizar_conexion(self)
    # \brief Actualiza el boton de conexion a estado conectado.
    def actualizar_conexion(self):
        self.connectButton.setText(QtGui.QApplication.translate("SGuiAlumno", "Conectar"))
        self.connectButton.setIcon(QtGui.QIcon(":/common/icon/curve1.png"))
        self.connectButton.setToolTip("Conectar")

    ## \fn actualizar_desconexion(self)
    # \brief Actualiza el boton de conexion a estado desconectado.
    def actualizar_desconexion(self):
        self.connectButton.setText(QtGui.QApplication.translate("SGuiAlumno", "Desconectar"))
        self.connectButton.setIcon(QtGui.QIcon(":/common/icon/symbol21.png"))
        self.connectButton.setToolTip("Desconectar")

    ## \fn realizar_conexion_cs(self)
    # \brief Manejador para la conexion y desconexion del usuario al grupo seleccionado.
    def realizar_conexion_cs(self):
        item = self.item_grupo_selected()
        if self.status:
            if item is not None:
                self.core_handlers["REQUEST_JOIN_MUC"](item.muc_jid)
                self.status = False
                self.btnHelp.setEnabled(True)
            else:
                self._message(u"Debes seleccionar un grupo para conectar.","critical")
                self.status = True
        else:
            self.core_handlers["REQUEST_EXIT_MUC"]()
            self.gui_cEditor_cliente.stop_btnColaborativo()
            self.status = True
            self.btnHelp.setEnabled(False)

    ## \fn _item_activated(self, item)
    # \brief Handle event for a list item activated (enter, double click, etc.)
    # \param item Item wich have been activated.
    def _item_activated(self, item):
        self.realizar_conexion_cs()

    def item_grupo_selected(self):
        item = self.ResourceListWidget.currentItem()
        if item is not None:
            row = self.ResourceListWidget.row(item)
            item = self.ResourceListWidget.item(row)
        return item

    #Metodo para mostrar mensaje de informativos
    def _message(self, msg, tipo):
        if tipo == "information":
            QtGui.QMessageBox.information(self, u"Información", msg)
        elif tipo == "about":
            QtGui.QMessageBox.about(self, u"Acerca de HGR", msg)
        elif tipo == "warning":
            QtGui.QMessageBox.warning(self, u"Atención", msg, u"Cerrar")
        elif tipo == "critical":
            QtGui.QMessageBox.critical(self, u"Error", msg, u"Ok")
        elif tipo == "information-server":
            ventana_info = VentanaMensajeInfoClient(msg)
            ventana_info.exec_()
        elif tipo == "question":
            salida = QtGui.QMessageBox.question(self, u"Atención", msg, u"No", u"Sí", defaultButtonNumber = 1)
            return salida

    ## \fn closeEvent(self, event)
    # \brief Handle window close event.
    # \param event Event.
    def closeEvent(self, event):
        if event.spontaneous():
            event.ignore()
            self.iconize()
        else:
            self.core_handlers["EXIT_APPLICATION"]()   
            
    ## \fn _hide_sharedclient(self, *args)
    # \brief Handle window close event shared.
    # \param *args.
    def _hide_sharedclient(self, *args):
        if self.shared_win:
            try:
                self.shared_win.hide()
            except Exception, e :
                print "Error al cerrar:", e

    ## \fn solicitarAyuda(self)
    # \brief metodo que solicita la ayuda al servidor.
    def solicitarAyuda(self):
        self.btnHelp.setEnabled(False)
        self.cambiarNombre.setEnabled(False)
        self.core_handlers["SOLICITAR_AYUDA_SERVIDOR"]()

if __name__=="__main__":
    app = QtGui.QApplication(sys.argv)
    app.setQuitOnLastWindowClosed(False)
    s = SGuiAlumno()
    sys.exit(app.exec_())
