#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from PyQt4 import QtCore, QtGui

from hga.controlgrupos.common.user_info import get_user_name
from hga.controlgrupos.common.settings import VALID_CHARS

import hga.gui.common.icon_rc

class identifyWindow(QtGui.QDialog):
    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.settings=QtCore.QSettings()
        self.center()
        label=QtGui.QLabel(u"""Por favor, introduce tu nombre y apellidos.""")
        self.tNombre=QtGui.QLineEdit()
        self.tNombre.setMaxLength(25)
        self.tNombre.setText(get_user_name())
        self.bAceptar=QtGui.QPushButton("Aceptar")
        self.bAceptar.setEnabled(False)

        self.connect(self.bAceptar, QtCore.SIGNAL("clicked()"), self.enterUserName)
        self.tNombre.textChanged.connect(self.habilitarAceptar)

        self.setWindowModality(QtCore.Qt.ApplicationModal)
        layout=QtGui.QVBoxLayout()
        layout.addWidget(label)
        layout.addWidget(self.tNombre)
        layout.addWidget(self.bAceptar)
        self.setLayout(layout)
        self.setWindowIcon(QtGui.QIcon(":/common/icon/avatar.png"))

        self.setWindowTitle(u"Identificación")
    
    def enterUserName(self):
        username=str(self.tNombre.text().toUtf8())
        username = self.sanitize_name(username)
        if len(username.strip(' ')) > 0:
            self.setUserName(username)
            self.accept()

    def setUserName(self, username):
        self.settings.setValue("user/name", QtCore.QVariant(username))

    def habilitarAceptar(self):
        username = str(self.tNombre.text().toUtf8())
        username = self.sanitize_name(username)
        if len(username.strip(' ')) > 0 and username != get_user_name():
            self.bAceptar.setEnabled(True)
        else:
            self.bAceptar.setEnabled(False)

    def center(self):
        GeoFrame = self.frameGeometry()
        center = QtGui.QDesktopWidget().availableGeometry().center()
        GeoFrame.moveCenter(center)
        self.move(GeoFrame.topLeft())


    def sanitize_name(self, data):
        data = data.lstrip()
        for a,b in ( ('á', 'a'), ('é', 'e'), ('í', 'i'), ('ó', 'o'), ('ú', 'u'), ('ñ', 'n'),
                    ('Á', 'A'), ('É', 'E'), ('Í', 'I'), ('Ó', 'O'), ('Ú', 'U'), ('Ñ', 'N') ):
            data=data.replace(a, b)
        data = str(filter(lambda k: k in VALID_CHARS, data))
        return data


if __name__=="__main__":
    import sys
    app=QtGui.QApplication(sys.argv)
    app.setOrganizationName("hgr")
    app.setApplicationName("Gestion")
    v=identifyWindow()
    v.show()
    app.exec_()
    